/*
  Copyright (C) 2000-2003 SKYRIX Software AG

  This file is part of OGo

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/
// $Id: OGoFileManagerFactory.m,v 1.3 2003/11/14 15:55:39 helge Exp $

#import "common.h"
#import "SkyFileManagerFactory.h"

@implementation SkyFileManagerFactory

static id sharedFactory = nil;

+ (id)sharedFileManagerFactory {
  if (sharedFactory)
    return sharedFactory;
  sharedFactory = [[OGoFileManagerFactory alloc] init];
  return sharedFactory;
}
- (id)init {
  if ((self = [super init])) {
    /* TODO: improve bundle loading! */
    static BOOL LoadBundles = YES;
    if (LoadBundles) {
      NGBundleManager *bm;
      NSBundle        *bundle;

      bm = [NGBundleManager defaultBundleManager];

      bundle = [bm bundleWithName:@"OGoFileSystemProject" type:@"ds"];
      bundle = [bm bundleWithName:@"OGoDatabaseProject"   type:@"ds"];
      LoadBundles = NO;
    }
  }
  return self;
}

- (id)fileManagerInContext:(id)_context forProjectGID:(EOGlobalID *)_gid {
  id project;

  if (_context == nil) {
    [self logWithFormat:@"ERROR: missing context argument!"];
    return nil;
  }
  if (_gid == nil) {
    [self logWithFormat:@"WARNING: no GID for filemanager construction!"];
    return nil;
  }
  
  project = [_context runCommand:@"project::get-by-globalid",
                 @"gid", _gid, nil];

  if ([project isKindOfClass:[NSArray class]])
    project = [project lastObject];

  if (!project) {
    NSLog(@"ERROR[%s] missing project for globalid %@",
          __PRETTY_FUNCTION__, _gid);
    return nil;
  }
  return [SkyFileManagerFactory fileManagerInContext:_context
                                forProject:project];
}

- (Class)fileManagerClassForScheme:(NSString *)_scheme {
  Class fmClass;
  
  if ([_scheme isEqualToString:@"file"]) {
    fmClass = NSClassFromString(@"SkyFSFileManager");

    if (fmClass == nil) { /* try to load bundle */
      [[[NGBundleManager defaultBundleManager]
                         bundleForClassNamed:@"SkyFSFileManager"] load];

      fmClass = NSClassFromString(@"SkyFSFileManager");
    }
  }
  else if ([_scheme isEqualToString:@"subversion"])
    fmClass = NSClassFromString(@"SkySvnFileManager");
  else if ([_scheme isEqualToString:@"skyrix"])
    fmClass = NSClassFromString(@"SkyProjectFileManager");
  else {
    [self logWithFormat:@"WARNING(%s): unknown filemanager scheme '%@', "
	  @"trying SkyProjectFileManager !",
	  __PRETTY_FUNCTION__, _scheme];
    fmClass = NSClassFromString(@"SkyProjectFileManager");
  }
  return fmClass;
}

- (id)fileManagerInContext:(id)_context forProject:(id)_project {
  NSURL    *url;
  NSString *urlString, *scheme;
  id       fm;
  Class    fmClass = Nil;

  if (_context == nil) {
    [self logWithFormat:@"ERROR: missing context argument!"];
    return nil;
  }
  if (_project == nil) {
    [self logWithFormat:@"WARNING: no EO for filemanager construction!"];
    return nil;
  }
  
  urlString = [_project valueForKey:@"url"];
  
  url = (![urlString isNotNull])
    ? [SkyFileManagerFactory skyrixBaseURL]
    : [NSURL URLWithString:urlString];
  
  scheme = [url scheme];

  /* construct filemanager */
  
  if ((fmClass = [self fileManagerClassForScheme:scheme]) == Nil) {
    NSLog(@"ERROR[%s] found no filemanager class for scheme %@: %@", 
	  __PRETTY_FUNCTION__, scheme, url);
    return nil;
  }
  
  fm = [[fmClass alloc] initWithContext:_context 
			projectGlobalID:[_project globalID]];
  
  return [fm autorelease];
}

- (NSURL *)skyrixBaseURL {
  static NSURL *SKYBase = nil;
  NSString *skyid;
  NSString *urlstr;

  if (SKYBase)
    return SKYBase;
    
  skyid = [[NSUserDefaults standardUserDefaults] stringForKey:@"skyrix_id"];
    
  if (![skyid length]) {
      urlstr  = [NSString stringWithFormat:@"skyrix://%@/",
                          [[NSHost currentHost] name]];
  }
  else {
    urlstr  = [NSString stringWithFormat:@"skyrix://%@/%@/",
                          [[NSHost currentHost] name],
                          skyid];
  }
  SKYBase = [[NSURL alloc] initWithString:urlstr relativeToURL:nil];
  NSLog(@"SKYRiX BaseURL: '%@'", SKYBase);
  return SKYBase;
}

- (NSURL *)subversionBaseURL {
  static NSURL *SVNBase = nil;
  NSString *svnPath;

  if (SVNBase)
    return SVNBase;
    
  svnPath = [[NSUserDefaults standardUserDefaults]
                             stringForKey:@"SkySvnRepositoryPath"];


    if (![svnPath length]) {
      svnPath = [NSString stringWithFormat:@"subversion://%@/",
                         [[NSHost currentHost] name]];
    }
    else {
      svnPath = [NSString stringWithFormat:@"subversion://%@/%@/",
                          [[NSHost currentHost] name], svnPath];
    }
  SVNBase = [[NSURL alloc] initWithString:svnPath relativeToURL:nil];
  NSLog(@"SVN BaseURL: '%@'", SVNBase);
  return SVNBase;
}

- (NSURL *)fileSystemBaseURL {
  static NSURL *FSBase = nil;
  NSString *fsPath;

  if (FSBase)
    return FSBase;
    
  fsPath = [[NSUserDefaults standardUserDefaults]
                            stringForKey:@"SkyFSPath"];
  if (fsPath == nil) {
      NSFileManager *fm;
      BOOL          isDir;
      
      fsPath = [[[[NSProcessInfo processInfo] environment]
                                 objectForKey:@"GNUSTEP_USER_ROOT"]
                                 stringByAppendingPathComponent:
                                 @"SkyFileSystem"];
      fm = [NSFileManager defaultManager];

      if (![fm fileExistsAtPath:fsPath isDirectory:&isDir]) {
        if (![fm createDirectoryAtPath:fsPath attributes:nil]) {
          NSLog(@"ERROR[%s]: Couldn`t create directory for skyrix filesystem "
                @"at path %@", __PRETTY_FUNCTION__, fsPath);
          return nil;
        }
      }
      else {
        if (!isDir) {
          NSLog(@"ERROR[%s]: couldn`t create directory at path %@, file exist",
                __PRETTY_FUNCTION__, fsPath);
          return nil;
        }
      }
  }
  fsPath = [NSString stringWithFormat:@"file://%@", fsPath];
  FSBase = [[NSURL alloc] initWithString:fsPath relativeToURL:nil];
  NSLog(@"FS BaseURL: '%@'", FSBase);
  return FSBase;
}

- (NSURL *)newFileSystemURLWithContext:(id)_ctx {
  NSURL         *baseURL;
  EOEntity      *project;
  NSDictionary  *keyRow;
  NSString      *path;
  NSFileManager *fm;

  baseURL = [SkyFileManagerFactory fileSystemBaseURL];

  project = [[_ctx valueForKey:LSDatabaseKey] entityNamed:@"Project"];

  if (!project) {
    NSLog(@"ERROR[%s]: missing entity for Project", __PRETTY_FUNCTION__);
    return nil;
  }

  keyRow = [[[_ctx valueForKey:LSDatabaseChannelKey] adaptorChannel]
                   primaryKeyForNewRowWithEntity:project];

  path = [[baseURL path] stringByAppendingPathComponent:
                         [[keyRow objectForKey:@"projectId"]
                                  stringValue]];

  fm = [NSFileManager defaultManager];

  if ([fm fileExistsAtPath:path isDirectory:NULL]) {
    NSLog(@"ERROR[%s] path already exist ... %@",
          __PRETTY_FUNCTION__, path);
    return nil;
  }
  if (![fm createDirectoryAtPath:path attributes:nil]) {
    NSLog(@"ERROR[%s]: couldn`t create path for directory %@",
          __PRETTY_FUNCTION__, path);
    return nil;
  }
  NSLog(@"createDirectoryAtPath %@", path);
  
  return [NSURL URLWithString:[NSString stringWithFormat:@"file://%@",
                                        path]];
}

/* DEPRECATED: only for compatibility */

+ (id)fileManagerInContext:(id)_context forProject:(id)_p {
  return [[self sharedFileManagerFactory] 
                fileManagerInContext:_context forProject:_p];
}
+ (id)fileManagerInContext:(id)_ctx forProjectGID:(EOGlobalID *)_gid {
  return [[self sharedFileManagerFactory] 
                fileManagerInContext:_ctx forProjectGID:_gid];
}

+ (NSURL *)skyrixBaseURL {
  return [[self sharedFileManagerFactory] skyrixBaseURL];
}
+ (NSURL *)subversionBaseURL {
  return [[self sharedFileManagerFactory] subversionBaseURL];
}
+ (NSURL *)fileSystemBaseURL {
  return [[self sharedFileManagerFactory] fileSystemBaseURL];
}

+ (NSURL *)newFileSystemURLWithContext:(id)_ctx {
  return [[self sharedFileManagerFactory] newFileSystemURLWithContext:_ctx];
}

@end /* SkyFileManagerFactory */

@implementation OGoFileManagerFactory
@end /* OGoFileManagerFactory */
