/*
  Copyright (C) 2000-2003 SKYRIX Software AG

  This file is part of OGo

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/
// $Id: LSSetJobResourceAssignmentCommand.m,v 1.2 2003/07/11 16:35:52 jan Exp $

#include <LSFoundation/LSDBObjectNewCommand.h>

@class NSNumber;

@interface LSSetJobResourceAssignmentCommand : LSDBObjectNewCommand
{
@private
  NSArray  *resources;
}
@end

#import "common.h"

@interface LSSetJobResourceAssignmentCommand(PrivatMethods)
- (void)setResources:(NSArray*)_resources;
- (NSArray*)resources;

@end

@implementation LSSetJobResourceAssignmentCommand

#if !LIB_FOUNDATION_BOEHM_GC
- (void)dealloc {
  RELEASE(self->resources);
  [super dealloc];
}
#endif

- (BOOL)_resource:(id)_object isInAssignmentList:(NSArray *)_list {
  NSEnumerator *resourceEnum = [_list objectEnumerator];
  NSNumber     *resourceId   = [_object valueForKey:@"resourceId"];
  id            obj          = nil;

  while ((obj = [resourceEnum nextObject])) {
    if ([resourceId isEqual:[obj valueForKey:@"resourceId"]]) {
      [_object takeValue:[obj valueForKey:@"jobResourceAssignmentId"]
                  forKey:@"_id_"];
      return YES;
    }
  }
  return NO;
}

- (BOOL)_assignment:(id)_object isInResourceList:(NSArray *)_list {
  NSEnumerator *resourceEnum = [_list objectEnumerator];
  NSNumber     *resourceId   = [_object valueForKey:@"resourceId"];
  id            obj          = nil;

  while ((obj = [resourceEnum nextObject])) {
    if ([resourceId isEqual:[obj valueForKey:@"resourceId"]]) {
      
      return YES;
    }
  }
  return NO;
}


- (void)_removeOldAssignmentsInContext:(id)_context {
  NSArray      *oldAssignments = nil;
  NSEnumerator *listEnum       = nil;
  id            assignment     = nil;

  oldAssignments = [[self object] objectForKey:@"toResourceAssignment"];
  listEnum       = [oldAssignments objectEnumerator];

  while ((assignment = [listEnum nextObject])) {
    if (![self _assignment:assignment isInResourceList:self->resources]) {
      LSDBObjectDeleteCommand *dCmd = nil;

      dCmd = LSLookupCommandV(@"jobresourceassignment", @"delete",
                              @"object", assignment, nil);
      [dCmd setReallyDelete:YES];
      [dCmd runInContext:_context];
    }
  }
}

- (void)_saveAssignmentsInContext:(id)_context {
  id            jobId     = [[self object] valueForKey:@"jobId"];
  NSArray      *oldAssignments = nil;
  NSEnumerator *listEnum       = nil;
  id            newResource    = nil; 

  oldAssignments = [[self object] objectForKey:@"toResourceAssignment"];
  listEnum       = [self->resources objectEnumerator];

  while ((newResource = [listEnum nextObject])) {
    if (![self _resource:newResource isInAssignmentList:oldAssignments]) {
      LSRunCommandV(_context,
                  @"jobresourceassignment", @"new",
                  @"jobId",         jobId,
                  @"operativePart", [newResource valueForKey:@"operativePart"],
                  @"resourceId",    [newResource valueForKey:@"resourceId"],
                  nil);
    }
    else {
      LSRunCommandV(_context,
                  @"jobresourceassignment", @"set",
                  @"jobResourceAssignmentId", [newResource valueForKey:@"_id_"],
                  @"jobId",         jobId,
                  @"operativePart", [newResource valueForKey:@"operativePart"],
                  @"resourceId",    [newResource valueForKey:@"resourceId"],
                  nil);
    }
  }
  [self setReturnValue:[self object]];
}


- (void)_prepareForExecutionInContext:(id)_context {
}


- (void)_executeInContext:(id)_context {
  LSRunCommandV(_context, @"job", @"get",
                @"jobId", [[self object] valueForKey:@"jobId"], nil);
  [self _removeOldAssignmentsInContext:_context];
  [self _saveAssignmentsInContext:_context];
  LSRunCommandV(_context, @"job", @"get",
                @"jobId", [[self object] valueForKey:@"jobId"], nil);
}

- (NSString*)entityName {
  return @"Job";
}

// accessors

- (void)setResources:(NSArray*)_resources {
  ASSIGN(self->resources, _resources);
}
- (NSArray*)resources {
  return self->resources;
}

// key/value coding

- (void)takeValue:(id)_value forKey:(id)_key {
  if ([_key isEqualToString:@"resources"]) {
    [self setResources:_value];
    return;
  }
  else if ([_key isEqualToString:@"object"]) {
    [self setObject:_value];
    return;
  }
  [super takeValue:_value forKey:_key];
}

- (id)valueForKey:(id)_key {
  if ([_key isEqualToString:@"resources"]) {
    return [self resources];
  }
  else if ([_key isEqualToString:@"object"])
    return [self object];
  return [super valueForKey:_key];
}

@end
