/*
  Copyright (C) 2000-2003 SKYRIX Software AG

  This file is part of OGo

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/
// $Id: LSUpdateProcessesCommand.m,v 1.2 2003/07/11 16:35:51 jan Exp $ 

#include <LSFoundation/LSDBObjectSetCommand.h>

@class NSArray;

@interface LSUpdateProcessesCommand : LSDBObjectSetCommand
{
@private
  NSArray *predecessors;
}

@end

#import "common.h"

@interface LSUpdateProcessesCommand(PrivateMethodes)
- (void)_checkPredecessorsInContext:(id)_context;
@end

@implementation LSUpdateProcessesCommand

- (void)dealloc {
  [self->predecessors release];

  [super dealloc];
}

- (void)_prepareForExecutionInContext:(id)_context {
  id obj = [self object];

  RELEASE(self->predecessors); self->predecessors = nil;
  self->predecessors = LSRunCommandV(_context, @"job", @"get-predecessors",
                                     @"object", obj, nil);
  RETAIN(self->predecessors);
}

- (void)_executeInContext:(id)_context {
  [self _checkPredecessorsInContext:_context];
}

// record initializer

- (NSString *)entityName {
  return @"Job";
}

// --- LSUpdateProcessesCommand(PrivateMethodes) --------------------------

- (BOOL)_dateKeys:(id)_a key1:(NSString **)_key1 key2:(NSString **)_key2 {
  NSString *kind   = [_a valueForKey:@"assignmentKind"];   //"01_end_start"
  NSArray  *parts  = [kind componentsSeparatedByString:@"_"];

  if ([parts count] != 3) // should be ("01", "end", "start")
    return NO;

  *_key1 = [NSString stringWithFormat:@"%@Date", [parts objectAtIndex:1]];
  *_key2 = [NSString stringWithFormat:@"%@Date", [parts objectAtIndex:2]];
  
  return YES;
}

- (NSTimeInterval)_timeInterval:(id)_object {
  NSCalendarDate *startDate = [_object valueForKey:@"startDate"];
  NSCalendarDate *endDate   = [_object valueForKey:@"endDate"];

  return [endDate timeIntervalSinceDate:startDate];
}

- (id)_getParentOf:(id)_obj inContext:(id)_context {
  id assign = [LSRunCommandV(_context, @"job", @"get-parentprocess",
                             @"object", _obj, nil) lastObject];

  return [assign valueForKey:@"toParentJob"];
}

- (id)_getChildsOf:(id)_obj inContext:(id)_context {
  id assigns = LSRunCommandV(_context, @"job", @"get-subprocesses",
                            @"object", _obj, nil);

  return [assigns valueForKey:@"toChildJob"];
}


- (void)_updateChildsOf:(id)_parent with:(NSTimeInterval)_deltaT
              inContext:(id)_context {
  if (_deltaT != 0) {
    NSArray *childs = [self _getChildsOf:_parent inContext:_context];
    int      i, cnt = [childs count];

    for (i=0; i<cnt; i++) {
      id              child     = [childs objectAtIndex:i];
      NSCalendarDate *startDate = [child valueForKey:@"startDate"];
      NSCalendarDate *endDate   = [child valueForKey:@"endDate"];

      startDate = [startDate addTimeInterval:_deltaT];
      endDate   = [endDate   addTimeInterval:_deltaT];

      [child takeValue:startDate forKey:@"startDate"];
      [child takeValue:startDate forKey:@"endDate"];

      LSRunCommandV(_context, @"job", @"set", @"object", child, nil);
      [self _updateChildsOf:child with:_deltaT inContext:_context];
    }
  }
}

- (void)_checkPredecessorsInContext:(id)_context {
  id             obj       = [self object];
  NSTimeInterval duration  = [self _timeInterval:obj];
  NSCalendarDate *oldStart = [obj valueForKey:@"startDate"];
  NSString       *key1, *key2;
  int            i, cnt    = [self->predecessors count];
  BOOL           changed   = NO;
  
  for (i=0; i<cnt; i++) {
    NSCalendarDate *startDate = nil;
    NSCalendarDate *endDate   = nil;
    id assign = [self->predecessors objectAtIndex:i];
    id pred   = [assign valueForKey:@"toParentJob"];

    [self _dateKeys:assign key1:&key1 key2:&key2];
    
    if ([key2 isEqualToString:@"startDate"]) {
      startDate = [pred valueForKey:key1];
      endDate   = [startDate addTimeInterval:duration];
    }
    else if ([key2 isEqualToString:@"endDate"]) {
      endDate   = [pred valueForKey:key1];
      startDate = [endDate addTimeInterval:-duration];
    }
    else
      NSAssert1(NO, @"key = '%@' must be 'startDate' or 'endDate'", key2);
      
    [obj takeValue:startDate forKey:@"startDate"];
    [obj takeValue:endDate   forKey:@"endDate"];
    changed = YES;
  }

  if (!changed) {
    id parent    = [self _getParentOf:obj inContext:_context];
    id startDate = [parent valueForKey:@"startDate"];
    
    startDate = [startDate earlierDate:[obj valueForKey:@"startDate"]];

    if (![startDate isEqual:[obj valueForKey:@"startDate"]]) {
      NSCalendarDate *endDate = [startDate addTimeInterval:duration];

      [obj takeValue:startDate forKey:@"startDate"];
      [obj takeValue:endDate   forKey:@"endDate"];
      changed = YES;
    }
  }
  
  if (changed) {
    NSTimeInterval deltaT = [[obj valueForKey:@"startDate"]
                                  timeIntervalSinceDate:oldStart];
    
    LSRunCommandV(_context, @"job", @"set", @"object", obj, nil);

    [self _updateChildsOf:obj with:deltaT inContext:_context];
  }
}

@end
