/*
  Copyright (C) 2000-2003 SKYRIX Software AG

  This file is part of OGo

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/
// $Id: LSSetResourceAssignmentCommand.m,v 1.2 2003/07/11 16:36:35 jan Exp $

#import "common.h"
#import "LSSetResourceAssignmentCommand.h"

@interface LSSetResourceAssignmentCommand(PrivatMethods)
- (void)setSubResources:(NSArray*)_subResources;
- (NSArray*)subResources;

@end

@implementation LSSetResourceAssignmentCommand

#if !LIB_FOUNDATION_BOEHM_GC
- (void)dealloc {
  RELEASE(self->subResources);
  [super dealloc];
}
#endif

- (BOOL)_resource:(id)_resource isInAssignmentList:(NSArray *)_list {
  NSEnumerator *assignmentEnum = [_list objectEnumerator];
  NSNumber     *resourceId     = [_resource valueForKey:@"resourceId"];
  id            myAssignment   = nil;

  while ((myAssignment = [assignmentEnum nextObject])) {
    if ([resourceId isEqual:[myAssignment valueForKey:@"subResourceId"]])
      return YES;
  }
  return NO;
}

- (BOOL)_assignment:(id)_assignment isInResourceList:(NSArray *)_list {
  NSEnumerator *resourceEnum = [_list objectEnumerator];
  NSNumber     *resourceId   = [_assignment valueForKey:@"subResourceId"];
  id            myResource   = nil;

  while ((myResource = [resourceEnum nextObject])) {
    if ([resourceId isEqual:[myResource valueForKey:@"resourceId"]])
      return YES;
  }
  return NO;
}

- (void)_removeOldAssignmentsInContext:(id)_context {
  NSArray      *oldAssignments = nil;
  NSEnumerator *listEnum       = nil;
  id            assignment     = nil;

  oldAssignments = [[self object] objectForKey:@"toSubResourceAssignment"];
  listEnum       = [oldAssignments objectEnumerator];

  while ((assignment = [listEnum nextObject])) {
    if (![self _assignment:assignment isInResourceList:self->subResources]) {
      LSDBObjectDeleteCommand *dCmd = nil;

      dCmd = LSLookupCommandV(@"resourceassignment", @"delete",
                              @"object", assignment, nil);
      [dCmd setReallyDelete:YES];
      [dCmd runInContext:_context];
    }
  }
}

- (void)_saveAssignmentsInContext:(id)_context {
  id            resourceId     = [[self object] valueForKey:@"resourceId"];
  NSArray      *oldAssignments = nil;
  NSEnumerator *listEnum       = nil;
  id            newResource    = nil; 

  oldAssignments = [[self object] objectForKey:@"toSubResourceAssignment"];
  listEnum       = [self->subResources objectEnumerator];

  while ((newResource = [listEnum nextObject])) {
    if (![self _resource:newResource isInAssignmentList:oldAssignments]) {
      LSRunCommandV(_context,
                @"resourceassignment", @"new",
                @"superResourceId",    resourceId,
                @"subResourceId",      [newResource valueForKey:@"resourceId"],
                nil);
    }
  }
  [self setReturnValue:[self object]];
}


- (void)_prepareForExecutionInContext:(id)_context {
}


- (void)_executeInContext:(id)_context {
  [self _removeOldAssignmentsInContext:_context];
  [self _saveAssignmentsInContext:_context];
}

- (NSString*)entityName {
  return @"Resource";
}

// accessors

- (void)setSubResources:(NSArray*)_subResources {
  ASSIGN(self->subResources, _subResources);
}
- (NSArray*)subResources {
  return self->subResources;
}

// key/value coding

- (void)takeValue:(id)_value forKey:(id)_key {
  if ([_key isEqualToString:@"subResources"]) {
    [self setSubResources:_value];
    return;
  }
  else if ([_key isEqualToString:@"object"]) {
    [self setObject:_value];
    return;
  }
  [super takeValue:_value forKey:_key];
}

- (id)valueForKey:(id)_key {
  if ([_key isEqualToString:@"subResources"]) {
    return [self subResources];
  }
  else if ([_key isEqualToString:@"object"])
    return [self object];
  return [super valueForKey:_key];
}

@end
