/*
  Copyright (C) 2000-2003 SKYRIX Software AG

  This file is part of OGo

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/
// $Id: LSSetResourceCommand.m,v 1.2 2003/07/11 16:36:35 jan Exp $

#import "common.h"
#import "LSSetResourceCommand.h"

@implementation LSSetResourceCommand

#if !LIB_FOUNDATION_BOEHM_GC
- (void)dealloc {
  RELEASE(self->subResources);
  [super dealloc];
}
#endif

- (void)_prepareForExecutionInContext:(id)_context {
  [self assert:([self valueForKey:@"resourceName"] != nil)
        reason:@"No resourceName set!"];
  [self assert:([self valueForKey:@"type"] != nil)
        reason:@"No type set!"];
  [super _prepareForExecutionInContext:_context];
}

- (void)_executeInContext:(id)_context {
  id obj = [self object];
  
  [self assert:(obj != nil) reason:@"no resource to act on!"];

  [super _executeInContext:_context];

   // if resource unit or group
  if ([[obj valueForKey:@"type"] intValue] > 1 && self->subResources != nil) {
    LSRunCommandV(_context, @"resource", @"set-subresources",
                  @"subResources", self->subResources,
                  @"object", [self object],
                  nil);
  }
}

- (NSString*)entityName {
  return @"Resource";
}

// accessors

- (NSArray*)subResources {
  return self->subResources;
}
- (void)setSubResources:(NSArray*)_subResources {
  ASSIGN(self->subResources, _subResources);
}

// key/value coding

- (void)takeValue:(id)_value forKey:(id)_key {
  if ([_key isEqualToString:@"subResources"]) {
    [self setSubResources:_value];
    return;
  }
  [super takeValue:_value forKey:_key];
}

- (id)valueForKey:(id)_key {
  if ([_key isEqualToString:@"subResources"])
    return [self subResources];
  return [super valueForKey:_key];
}

@end
