/*
  Copyright (C) 2000-2003 SKYRIX Software AG

  This file is part of OGo

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/
// $Id: LSGetMemberForTeamCommand.m,v 1.3 2003/11/19 16:18:08 helge Exp $

#include "LSGetMemberForCompanyCommand.h"

@interface LSGetMemberForTeamCommand : LSGetMemberForCompanyCommand
@end

#import "common.h"

@implementation LSGetMemberForTeamCommand

static int compareAccounts(id member1, id member2, void *context) {
  NSString *name1 = [member1 valueForKey:@"login"];
  NSString *name2 = [member2 valueForKey:@"login"];
  
  if (name1 == nil) name1 = @"";
  if (name2 == nil) name2 = @"";

  return [name1 compare:name2];
}

/* record initializer */

- (NSString *)entityName {
  return @"Team";
}

- (NSString *)memberEntityName {
  return @"Person";
}

- (void)_executeInContext:(id)_context {
  id  obj = nil;
  id  team;
  NSEnumerator *e;
  
  [super _executeInContext:_context];
  
  obj = [self groups];
  e   = [obj objectEnumerator];
  
  while ((team = [e nextObject])) {
    NSMutableArray *m;
      
    if ((m = [team valueForKey:@"members"]) == nil)
      continue;
      
#if DEBUG
    [self assert:(![m containsObject:[NSNull null]])
          format:@"invalid members %@ of team: %@", m, team];
#endif
    if (![m isKindOfClass:[NSMutableArray class]])
      m = [m mutableCopy];
    else
      m = [m retain];
    
    [m sortUsingFunction:compareAccounts context:self];
        
    /* get extended attributes  */
    LSRunCommandV(_context, @"person", @"get-extattrs",
                  @"objects", m,
                  @"relationKey", @"companyValue", nil);
      
    /* get telephones */
    LSRunCommandV(_context, @"person", @"get-telephones",
                  @"objects", m,
                  @"relationKey", @"telephones", nil);
    
    [m release];
  }
}

/* key/value coding */

- (void)takeValue:(id)_value forKey:(id)_key {
  if ([_key isEqualToString:@"team"] || [_key isEqualToString:@"object"]) {
    [self setGroup:_value];
    return;
  }
  else if ([_key isEqualToString:@"teams"]) {
    [self setGroups:_value];
    return;
  }
  [super takeValue:_value forKey:_key];
}

- (id)valueForKey:(id)_key {
  if ([_key isEqualToString:@"team"] || [_key isEqualToString:@"object"])
    return [self group];
  else if ([_key isEqualToString:@"teams"])
    return [self groups];
  return [super valueForKey:_key];
}

@end /* LSGetMemberForTeamCommand */
