/*
  Copyright (C) 2000-2003 SKYRIX Software AG

  This file is part of OGo

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/
// $Id: bmlookup.m,v 1.2 2003/07/11 17:12:50 jan Exp $

#include "common.h"
#import <EOControl/EOControl.h>
#include <NGExtensions/NSFileManager+Extensions.h>
#include <NGExtensions/NGFileFolderInfoDataSource.h>
#include <NGExtensions/NGExtensions.h>

int main(int argc, char **argv, char **env) {
  NSAutoreleasePool *pool;
  NSArray           *args;
  NGBundleManager   *bm;
  
  pool = [[NSAutoreleasePool alloc] init];
#if LIB_FOUNDATION_LIBRARY
  [NSProcessInfo initializeWithArguments:argv count:argc environment:env];
#endif
  bm   = [NGBundleManager defaultBundleManager];
  
  args = [[NSProcessInfo processInfo] arguments];
  if ([args count] < 2) {
    NSLog(@"usage: %@ type name", [[args objectAtIndex:0] lastPathComponent]);
    exit(1);
  }
  else if ([args count] == 2) {
    NSString *btype;
    NSEnumerator *resources;
    id resource;
    
    btype = [args objectAtIndex:1];
    
    NSLog(@"lookup type=%@ ...", btype);
    
    resources = [[bm providedResourcesOfType:btype] objectEnumerator];
    while ((resource = [resources nextObject])) {
      NSString *rname;
      NSBundle *bundle;
      
      rname = [resource objectForKey:@"name"];
      bundle = [bm bundleProvidingResource:rname ofType:btype];
      
      NSLog(@"  resource(bundle=%@): %@", [bundle bundlePath], resource);
    }
  }
  else {
    NSString *btype, *bname;
    NSBundle *bundle;
    
    btype = [args objectAtIndex:1];
    bname = [args objectAtIndex:2];
    
    NSLog(@"lookup type=%@ name=%@ ...", btype, bname);
    
    bundle = [bm bundleProvidingResource:bname ofType:btype];
    NSLog(@"  bundle: %@", bundle);
    
    if ([[NSUserDefaults standardUserDefaults] boolForKey:@"load"]) {
      if (![bundle load])
        NSLog(@"Couldn't load bundle: %@", bundle);
      else
        NSLog(@"Loaded bundle: %@", bundle);
    }
  }
  
  [pool release];
  
  exit(0);
  return 0;
}
