/*
  Copyright (C) 2000-2003 SKYRIX Software AG

  This file is part of OGo

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/
// $Id: WOComponent+JS.m,v 1.2 2003/07/11 17:12:51 jan Exp $

#include "common.h"
#include <NGObjWeb/NGObjWeb.h>

/*
  WOComponent JavaScript object

  Properties

    String      sessionID
    String      name
    String      path
    String      baseURL
    Object      context
    Object      session
    Object      application
    WOComponent parent
    bool        hasSession
    bool        cachingEnabled
    bool        isEventLoggingEnabled
    bool        isStateless
    bool        synchronizesVariablesWithBindings
  
  Methods

    reset()    
    WOComponent     pageWithName(name)
    WOElement       templateWithName(name)
    Object          performParentAction(name)
    bool            canGetValueForBinding(name)
    bool            canSetValueForBinding(name)
                    setValueForBinding(value,name)
    Object          valueForBinding(name)
    bool            hasBinding(name)
                    print(string[,...string])
    ResourceManager getResourceManager()
*/

static NSNumber *nYes = nil;
static NSNumber *nNo  = nil;

#define ENSURE_BOOLNUMS {\
  if (nYes == nil) nYes = [[NSNumber alloc] initWithBool:YES];\
  if (nNo  == nil) nNo  = [[NSNumber alloc] initWithBool:NO];\
}

@implementation WOComponent(JSKVC)

#if 1
- (void)takeValue:(id)_value forJSPropertyNamed:(NSString *)_key {
  [self takeValue:_value forKey:_key];
}
- (id)valueForJSPropertyNamed:(NSString *)_key {
  return [self valueForKey:_key];
}
#endif

@end /* WOComponent(JSKVC) */

@implementation WOComponent(JSFunctions)

- (id)_jsfunc_reset:(NSArray *)_args {
  [self reset];
  return self;
}

- (id)_jsfunc_pageWithName:(NSArray *)_args {
  return [self pageWithName:[[_args objectAtIndex:0] stringValue]];
}
- (id)_jsfunc_templateWithName:(NSArray *)_args {
  return [self templateWithName:[[_args objectAtIndex:0] stringValue]];
}
- (id)_jsfunc_performParentAction:(NSArray *)_args {
  return [self performParentAction:[[_args objectAtIndex:0] stringValue]];
}

- (id)_jsfunc_canGetValueForBinding:(NSArray *)_args {
  ENSURE_BOOLNUMS;
  return [self canGetValueForBinding:[[_args objectAtIndex:0] stringValue]]
    ? nYes : nNo;
}
- (id)_jsfunc_canSetValueForBinding:(NSArray *)_args {
  ENSURE_BOOLNUMS;
  return [self canSetValueForBinding:[[_args objectAtIndex:0] stringValue]]
    ? nYes : nNo;
}

- (id)_jsfunc_setValueForBinding:(NSArray *)_args {
  [self setValue:[_args objectAtIndex:0]
        forBinding:[[_args objectAtIndex:1] stringValue]];
  return self;
}
- (id)_jsfunc_valueForBinding:(NSArray *)_args {
  return [self valueForBinding:[[_args objectAtIndex:0] stringValue]];
}

- (id)_jsfunc_hasBinding:(NSArray *)_args {
  ENSURE_BOOLNUMS;
  return [self hasBinding:[[_args objectAtIndex:0] stringValue]]
    ? nYes : nNo;
}

- (id)_jsfunc_getResourceManager:(NSArray *)_args {
  return [self resourceManager];
}

- (id)_jsfunc_print:(NSArray *)_args {
  NSEnumerator    *e;
  id              o;
  BOOL            isFirst;
  NSMutableString *ms;
  
  isFirst = YES;
  ms = [NSMutableString stringWithCapacity:128];
  
  e = [_args objectEnumerator];
  while ((o = [e nextObject])) {
    NSString *s;
    
    if (!isFirst) [ms appendString:@" "];
    else isFirst = NO;
    
    s = [o stringValue];
    [ms appendString:s];
  }
  
  [self logWithFormat:@"%@", ms];
  
  return self;
}

@end /* WOComponent(JSFunctions) */

@implementation WOComponent(JSProperties)

- (id)_jsprop_sessionID {
  return [[self session] sessionID];
}
- (id)_jsprop_name {
  return [self name];
}
- (id)_jsprop_path {
  return [self path];
}
- (id)_jsprop_baseURL {
  return [self baseURL];
}

- (id)_jsprop_context {
  return [self context];
}
- (id)_jsprop_session {
  return [self session];
}
- (id)_jsprop_application {
  return [self application];
}
- (id)_jsprop_parent {
  return [self parent];
}

- (id)_jsprop_hasSession {
  ENSURE_BOOLNUMS;
  return [self hasSession] ? nYes : nNo;
}

- (void)_jsprop_cachingEnabled:(id)_value {
  [self setCachingEnabled:[_value boolValue]];
}
- (id)_jsprop_cachingEnabled {
  ENSURE_BOOLNUMS;
  return [self isCachingEnabled] ? nYes : nNo;
}

- (id)_jsprop_isEventLoggingEnabled {
  ENSURE_BOOLNUMS;
  return [self isEventLoggingEnabled] ? nYes : nNo;
}
- (id)_jsprop_isStateless {
  ENSURE_BOOLNUMS;
  return [self isStateless] ? nYes : nNo;
}
- (id)_jsprop_synchronizedVariablesWithBindings {
  ENSURE_BOOLNUMS;
  return [self synchronizesVariablesWithBindings] ? nYes : nNo;
}

@end /* WOComponent(JSProperties) */
