/*
  Copyright (C) 2000-2003 SKYRIX Software AG

  This file is part of OGo

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/
// $Id: _WOStringTable.m,v 1.2 2003/07/11 17:12:51 jan Exp $

#include "_WOStringTable.h"
#include "common.h"

@implementation _WOStringTable

- (id)initWithPath:(NSString *)_path {
  self->path = [_path copyWithZone:[self zone]];
  return self;
}

#if !LIB_FOUNDATION_BOEHM_GC
- (void)dealloc {
  RELEASE(self->path);
  RELEASE(self->lastRead);
  RELEASE(self->data);
  [super dealloc];
}
#endif

// loading

- (void)checkState {
  if (self->data == nil) {
    NSString     *tmp;
    NSDictionary *plist;
    NSString *fmt;
    
    if ((tmp = [NSString stringWithContentsOfFile:self->path]) == nil) {
      self->data = nil;
      return;
    }
    
    fmt = @"%s: could not load strings file '%@': %@";
    
    NS_DURING {
      if ((plist = [tmp propertyListFromStringsFileFormat]) == nil) {
        NSLog(@"%s: could not load strings file '%@'",
              __PRETTY_FUNCTION__,
              self->path);
      }
      self->data = [plist copy];
    }
    NS_HANDLER {
      NSLog(fmt, __PRETTY_FUNCTION__, self->path, localException);
      self->data = nil;
    }
    NS_ENDHANDLER;
  }
}

/* access */

- (NSString *)stringForKey:(NSString *)_key withDefaultValue:(NSString *)_def {
  NSString *value;
  [self checkState];
  value = [self->data objectForKey:_key];
  return value ? value : _def;
}

@end /* _WOStringTable */
