/* 
   PostgreSQL72Adaptor.m

   Copyright (C) 1999 MDlink online service center GmbH and Helge Hess

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the PostgreSQL72 Adaptor Library

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
// $Id: PostgreSQL72Adaptor.m,v 1.1.1.1 2003/07/09 22:57:43 cvs Exp $

#import "common.h"
#import "PostgreSQL72Adaptor.h"
#import "PostgreSQL72Context.h"
#import "PostgreSQL72Channel.h"
#import "PostgreSQL72Expression.h"
#import "PostgreSQL72Values.h"

@implementation PostgreSQL72Adaptor

- (id)initWithName:(NSString *)_name {
  if ((self = [super initWithName:_name])) {
  }
  return self;
}

- (void)gcFinalize {
}

#if !LIB_FOUNDATION_BOEHM_GC
- (void)dealloc {
  //  NSLog(@"%s", __PRETTY_FUNCTION__);
  [self gcFinalize];
  [super dealloc];
}
#endif

/* NSCopying methods */

- (id)copyWithZone:(NSZone *)_zone {
  return RETAIN(self);
}

// connections

- (NSString *)serverName {
  NSString *serverName;

  serverName = [[self connectionDictionary] objectForKey:@"hostName"];

  if (serverName == nil) { // lookup env-variable
    serverName = [[[NSProcessInfo processInfo] environment]
                                  objectForKey:@"PGHOST"];
  }
  
  return AUTORELEASE([serverName copy]);
}
- (NSString *)loginName {
  return AUTORELEASE([[[self connectionDictionary]
                             objectForKey:@"userName"] copy]);
}
- (NSString *)loginPassword {
  return AUTORELEASE([[[self connectionDictionary]
                             objectForKey:@"password"] copy]);
}
- (NSString *)databaseName {
  return AUTORELEASE([[[self connectionDictionary]
                             objectForKey:@"databaseName"] copy]);
}

- (NSString *)port {
  return AUTORELEASE([[[self connectionDictionary] objectForKey:@"port"] copy]);
}
- (NSString *)options {
  return AUTORELEASE([[[self connectionDictionary]
                             objectForKey:@"options"] copy]);
}
- (NSString *)tty {
  return AUTORELEASE([[[self connectionDictionary] objectForKey:@"tty"] copy]);
}

/* sequence for primary key generation */

- (NSString *)primaryKeySequenceName {
  NSString *seqName;

  seqName =
    [[self pkeyGeneratorDictionary] objectForKey:@"primaryKeySequenceName"];
  return [[seqName copy] autorelease];
}

- (NSString *)newKeyExpression {
  NSString *newKeyExpr;
  
  newKeyExpr =
    [[self pkeyGeneratorDictionary] objectForKey:@"newKeyExpression"];
  return [[newKeyExpr copy] autorelease];
}

// formatting

- (NSString *)charConvertExpressionForAttributeNamed:(NSString *)_attrName {
  return _attrName;
}

- (id)formatValue:(id)value forAttribute:(EOAttribute *)attribute {
  NSString *result;
  
  result = [value stringValueForPostgreSQLType:[attribute externalType]
                  attribute:attribute];

  //NSLog(@"formatting value %@ result %@", value, result);
  //NSLog(@"  value class %@ attr %@ attr type %@",
  //             [value class], attribute, [attribute externalType]);

  return result;
}

- (BOOL)attributeAllowedInDistinctSelects:(EOAttribute *)_attr {
  return YES;
}

/* types */

- (BOOL)isValidQualifierType:(NSString *)_typeName {
  return YES;
}

/* adaptor info */

- (Class)adaptorContextClass {
  return [PostgreSQL72Context class];
}
- (Class)adaptorChannelClass {
  return [PostgreSQL72Channel class];
}

- (Class)expressionClass {
  return [PostgreSQL72Expression class];
}

@end /* PostgreSQL72Adaptor */

void __linkPostgreSQL72Adaptor(void) {
  extern void __link_EOAttributePostgreSQL72();
  extern void __link_NSStringPostgreSQL72();
  extern void __link_PostgreSQL72ChannelModel();
  extern void __link_PostgreSQL72Values();
  ;
  [PostgreSQL72Channel    class];
  [PostgreSQL72Context    class];
  [PostgreSQL72Exception  class];
  [PostgreSQL72Expression class];
  __link_EOAttributePostgreSQL72();
  __link_NSStringPostgreSQL72();
  //__link_PostgreSQL72ChannelModel();
  __link_PostgreSQL72Values();
  __linkPostgreSQL72Adaptor();
}
