// $Id: nhsc.c,v 1.1.1.1 2003/07/09 22:57:44 cvs Exp $

/* pi-nredir.c: Redirect a connection over the network
 *
 * Copyright (C) 1997, Kenneth Albanowski
 *
 * This is free software, licensed under the GNU Public License V2.
 * See the file COPYING for details.
 */

#include <stdio.h>
#include <stdlib.h>
#ifdef __MINGW32__
#  include <windows.h>
#  include <winsock.h>
#endif
#include "pi-config.h"
#include <pi-source.h>
#include <pi-socket.h>
#include <pi-dlp.h>

#define DEFAULT_NETTIMEOUT 600

int main(int argc, char *argv[]) {
  struct pi_sockaddr addr2;
  struct pi_sockaddr addr;
  int    address;
  int sd, sd2;
  struct NetSyncInfo N;
  char buffer[0xffff];
  int len;
  int ret;

  int timeout = DEFAULT_NETTIMEOUT;

  if (argc < 2) {
    fprintf(stderr,"usage:%s %s [timeout-secs]\n", argv[0], TTYPrompt);
    exit(2);
  }

  if (argc > 2) {
    sscanf(argv[2], "%i", &timeout);
    if (timeout < 10) {
      timeout = 10;
      printf("timeout must be at least 10 seconds; setting to 10 seconds\n");
    }
    else {
      printf("setting timeout to %i seconds\n", timeout);
    }
  }
  timeout *= 1000; // milli secs

  //  sd2 = pi_socket(AF_INET, PI_SOCK_STREAM, PI_PF_PADP);
  sd2 = pi_socket(PI_AF_PILOT, PI_SOCK_STREAM, PI_PF_NET);
  if (sd2 < 0) {
    perror("socket");
    exit(1);
  }
  //fcntl(((struct pi_socket*)sd2)->sd, F_SETFL, O_NONBLOCK ); // no timeout

  if (!(sd = pi_socket(PI_AF_PILOT, PI_SOCK_STREAM, PI_PF_DLP))) {
    perror("pi_socket");
    exit(1);
  }
    
  addr.pi_family = PI_AF_PILOT;
  strcpy(addr.pi_device,argv[1]);
  
  ret = pi_bind(sd, (struct sockaddr*)&addr, sizeof(addr));
  if(ret == -1) {
    perror("pi_bind");
    exit(1);
  }
  
  ret = pi_listen(sd, 1);
  if(ret == -1) {
    perror("pi_listen");
    exit(1);
  }

  sd = pi_accept(sd, 0, 0);
  if(sd < 0) {
    perror("pi_accept");
    exit(1);
  }

  dlp_OpenConduit(sd);
  
  if (dlp_ReadNetSyncInfo(sd, &N) < 0) {
    char *eHostName, *eAddress, *eMask;
    
    eHostName = getenv("NHSD_HOSTNAME");
    eAddress  = getenv("NHSD_ADDRESS");
    eMask     = getenv("NHSD_NETMASK");
    
    N.lanSync  = 1;
    
    if ((eAddress == NULL) || (eAddress == NULL)) {
      fprintf(stderr, "Unable to read network information, cancelling sync.\n");
      fprintf(stderr, "You may want to set environment variables "
              "'NHSD_HOSTNAME' and 'NHSD_ADDRESS' !\n");
      exit(1);
    }
    
    strcpy(N.hostName,    eHostName);
    strcpy(N.hostAddress, eAddress);
    if (eMask) strcpy(N.hostSubnetMask, eMask);
    else strcpy(N.hostSubnetMask, "255.255.255.0");
  }
  
  if (!N.lanSync) {
    char *eHostName, *eAddress, *eMask;

    eHostName = getenv("NHSD_HOSTNAME");
    eAddress  = getenv("NHSD_ADDRESS");
    eMask     = getenv("NHSD_NETMASK");
    N.lanSync  = 1;

    if ((eHostName == NULL) || (eAddress == NULL)) {
      fprintf(stderr, "Unable to read network information, cancelling sync.\n");
      fprintf(stderr, "You may want to set environment variables "
              "'NHSD_HOSTNAME' and 'NHSD_ADDRESS' !\n");
      exit(1);
    }

    fprintf(stderr, "LAN Sync not enabled on your PalmPilot (ignored ..) !\n");

    strcpy(N.hostName,    eHostName);
    strcpy(N.hostAddress, eAddress);
    if (eMask) strcpy(N.hostSubnetMask, eMask);
    else strcpy(N.hostSubnetMask, "255.255.255.0");
  }
  putenv("PILOTLOGFILE=PiDebugNet.log");
  
  memset(&addr2, 0, sizeof(addr2));
  addr2.pi_family = PI_AF_PILOT;
  //addr2.sin_port = htons(14238);
  // not possible to modifie port .. (see @ libpisock)

  if ((address = inet_addr(N.hostAddress))==-1) {
    fprintf(stderr, "Unable to parse PC address '%s'\n", N.hostAddress);
    exit(1);
  }
  strcpy(addr2.pi_device, "net:");
  strcpy(addr2.pi_device+4, N.hostAddress);

  //printf("connecting %s:14238 .. ", N.hostAddress);
  printf("connecting %s:14238 .. ", addr2.pi_device);
  fflush(stdout);
 
  ret = pi_connect(sd2, (struct sockaddr*)&addr2, sizeof(addr2));
  if (ret<0) {
    perror("Unable to connect to PC");
    exit(1);
  }

  // setting network socket timeout
  len = sizeof(int);
  if (pi_setsockopt(sd2, 0, PI_DEV_TIMEOUT, &timeout, &len) == -1) {
    printf("error setting network socket timeout\n");
  }
 
  printf("connected, starting transfer .. "); fflush(stdout);
 
  while ((len = pi_read(sd2, buffer, 0xffff))>0) {
    //printf("  tx %i bytes\n", len);
    pi_write(sd, buffer, len);
    len = pi_read(sd, buffer, 0xffff);
    if (len < 0)
      break;
    pi_write(sd2, buffer, len);
  }
  printf("done.\n");

  {
    int state, size;
    state = PI_SOCK_CONEN;
    size = sizeof (state);
    pi_setsockopt (sd, PI_LEVEL_SOCK, PI_SOCK_STATE, &state, &size);
    pi_setsockopt (sd2, PI_LEVEL_SOCK, PI_SOCK_STATE, &state, &size);
  }
	
  pi_close(sd);
  pi_close(sd2);
  
  return 0;
}
