/*
  Copyright (C) 2000-2003 SKYRIX Software AG

  This file is part of OGo

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/
// $Id: SkyAccessList.m,v 1.2 2003/07/11 16:45:57 jan Exp $

/*
  >  searchPopupTitle     default = nil
  >  accesslistTitle      default =
  <> accessList           default = nil

  {
  personGID = 'rwx';
  teamGID   = 'i';
  }
*/

#import <OGoFoundation/OGoFoundation.h>
#import <LSFoundation/LSFoundation.h>
#import <WEExtensions/WEContextConditional.h>
#include <EOControl/EOControl.h>

@interface NSObject(Private)
- (EOGlobalID *)globalID;
@end /* NSObject(Private) */

@interface SkyAccessList : LSWComponent
{
  id                  team;
  id                  item;
  NSMutableDictionary *accessList;
  NSString            *searchString;
  BOOL                syncState;
  NSMutableArray      *companies;
  NSArray             *accessChecks;
  BOOL                isInTable;
  BOOL                isViewerMode;
  id                  accessItem;
  id                  myLabels;
}
@end /* SkyAccessList */

#include "common.h"

@implementation SkyAccessList

- (id)init {
  if ((self = [super init])) {
    self->syncState    = NO;
    self->isInTable    = NO;
    self->isViewerMode = NO;
  }
  return self;
}

#if !LIB_FOUNDATION_BOEHM_GC
- (void)dealloc {
  RELEASE(self->team);
  RELEASE(self->item);
  RELEASE(self->accessItem);
  RELEASE(self->searchString);
  RELEASE(self->accessList);
  RELEASE(self->companies);
  RELEASE(self->myLabels);
  [super dealloc];
}
#endif


static EOGlobalID *_globaID(id _obj) {
  if ([_obj isKindOfClass:[NSDictionary class]]) {
    return [_obj objectForKey:@"globalID"];
  }
  else {
    return [_obj globalID];
  }
}

- (void)_addObj:(id)_obj toCompanies:(NSMutableArray *)_companies {
  NSEnumerator *enumerator;
  id           obj;
  EOGlobalID   *gid;

  gid        = _globaID(_obj);
  enumerator = [_companies objectEnumerator];
  
  while ((obj = [enumerator nextObject])) {
    if ([_globaID(obj) isEqual:gid]) {
      break;
    }
  }
  if (!obj)
    [_companies addObject:_obj];
}

- (void)syncVars {
  if (!self->syncState) {
    NSEnumerator *enumerator;
    id           obj;

    if (self->isViewerMode) {
      RELEASE(self->companies); self->companies = nil;
    }
      
    
    if (self->accessList == nil)
      self->accessList = [[NSMutableDictionary alloc] init];
    
    self->syncState = YES;
      
    if (self->companies == nil) { /* first loop */
      NSArray *tmpArray;
      id      *agids, *tgids;
      int     aCnt, tCnt;

      self->companies = [[NSMutableArray alloc] initWithCapacity:64];
      aCnt            = [self->accessList count];
      agids           = malloc(sizeof(id) * aCnt);
      tgids           = malloc(sizeof(id) * aCnt);
      aCnt            = 0;
      tCnt            = 0;
      enumerator      = [self->accessList keyEnumerator];
        
      while ((obj = [enumerator nextObject])) {
        if ([[obj entityName] isEqualToString:@"Team"]) {
          tgids[tCnt++] = obj;
        }
        else if ([[obj entityName] isEqualToString:@"Person"]) {
          agids[aCnt++] = obj;
        }
        else {
          NSLog(@"WARNING[%s]: unexpected gid %@", __PRETTY_FUNCTION__, obj);
        }
      }
      if (tCnt > 0) {
        NSArray *a;
        
        tmpArray = [[NSArray alloc] initWithObjects:tgids count:tCnt];
        a        = [self runCommand:@"team::get-by-globalID",
                         @"gids", tmpArray, nil];
        
        [self->companies addObjectsFromArray:a];
        RELEASE(tmpArray); tmpArray = nil;
      }
      if (aCnt > 0) {
        NSArray *a;
        
        tmpArray = [[NSArray alloc] initWithObjects:agids count:aCnt];
        a        = [self runCommand:@"person::get-by-globalID",
                         @"gids", tmpArray, nil];
        [self->companies addObjectsFromArray:a];
        RELEASE(tmpArray); tmpArray = nil;
      }
      enumerator = [self->companies objectEnumerator];
      while ((obj = [enumerator nextObject])) {
        NSMutableDictionary *dict;
        NSString            *right;
        int                 rCnt, i;

        dict = [NSMutableDictionary dictionaryWithCapacity:16];
        [obj setObject:dict forKey:@"accessRights"];

        right = [self->accessList objectForKey:[obj globalID]];
        for (i=0, rCnt = [right cStringLength]; i < rCnt; i++) {
          [dict setObject:[NSNumber numberWithBool:YES]
                forKey:[right substringWithRange:NSMakeRange(i, 1)]];
        }
      }
      free(tgids); tgids = NULL;
      free(agids); agids = NULL;
    }
    else {
      /* remove empty entries from companies remove accessRights and sort */
      NSEnumerator *enumerator;
      NSArray      *tmpArray;
      id           *teams, *persons, obj;
      int          cCnt, i,tCnt, pCnt;

      tCnt    = [self->companies count];
      teams   = malloc(sizeof(id) * tCnt);
      persons = malloc(sizeof(id) * tCnt);
      tCnt    = 0;
      pCnt    = 0;

      for (i = 0,  cCnt = [self->companies count]; i < cCnt; i++) {
        NSMutableDictionary *dict;
        NSEnumerator        *keyEnum;
        id                  key;
        NSMutableString     *str;

        obj     = [self->companies objectAtIndex:i];
        dict    = [obj objectForKey:@"accessRights"];
        keyEnum = nil;
        
        if ([dict isNotNull])
          keyEnum = [dict keyEnumerator];
        
        str     = nil;
        
        while ((key = [keyEnum nextObject])) {
          id o;

          o = [dict objectForKey:key];

          if ([o boolValue]) {
            if (!str)
              str = [NSMutableString stringWithCapacity:16];
            
            [str appendString:key];
          }
        }
        if (!str) {
          [obj takeValue:[EONull null] forKey:@"accessRights"];
          [self->accessList removeObjectForKey:_globaID(obj)];
        }
        else { /* start sort and set obj in accessList */
          if ([[obj valueForKey:@"isTeam"] boolValue])
            teams[tCnt++] = obj;
          else
            persons[pCnt++] = obj;
          
          [self->accessList setObject:str forKey:_globaID(obj)];
        }
      }
      [self->companies removeAllObjects];
      
      tmpArray = [[NSArray alloc] initWithObjects:teams count:tCnt];
      [self->companies addObjectsFromArray:tmpArray];
      RELEASE(tmpArray);
      tmpArray = [[NSArray alloc] initWithObjects:persons count:pCnt];
      [self->companies addObjectsFromArray:tmpArray];
      RELEASE(tmpArray); tmpArray = NULL;
      
      free(teams);   teams   = NULL;
      free(persons); persons = NULL;

      if (![self->searchString isNotNull])
        self->searchString = @"";
      
      if ([self->searchString length]) {
        /* add search accounts to companies; set accessRights */
        id searchResult;

        searchResult = [self runCommand:
                             @"account::extended-search",
                             @"fetchGlobalIDs", [NSNumber numberWithBool:YES],
                             @"operator",       @"OR",
                             @"name",           self->searchString,
                             @"firstname",      self->searchString,
                             @"description",    self->searchString,
                             @"login",          self->searchString,
                             @"keywords",       self->searchString,
                             nil];
        searchResult = [self runCommand:@"person::get-by-globalid",
                             @"gids", searchResult,
                             @"groupBy", @"globalID",
                             @"attributes", [NSArray arrayWithObjects:@"name",
                                              @"firstname", @"login", nil],
                             nil];

        enumerator = [searchResult objectEnumerator];

        while ((obj = [enumerator nextObject])) {
          obj = [obj mutableCopy];
          AUTORELEASE(obj);
          [obj setObject:[NSMutableDictionary dictionary]
               forKey:@"accessRights"];

          [self _addObj:obj toCompanies:self->companies];
        }
        {
          NSArray *sA; 

          sA = [self->companies sortedArrayUsingKeyOrderArray:
                           [NSArray arrayWithObjects:
                             [EOSortOrdering sortOrderingWithKey:@"login"
                               selector:EOCompareAscending],
                             [EOSortOrdering sortOrderingWithKey:@"name"
                               selector:EOCompareAscending],
                             [EOSortOrdering sortOrderingWithKey:@"firstname"
                               selector:EOCompareAscending],
                                              nil]];
          self->companies = [sA mutableCopy];
        }
      }
      else if (self->team) {
        /* add team entries to companies; set accessRights */
        id searchResult;

        if (![[self->team valueForKey:@"accessRights"] isNotNull])
          [self->team takeValue:[NSMutableDictionary dictionary]
               forKey:@"accessRights"];

        if (![self->companies containsObject:self->team])
          [self->companies addObject:self->team];

        searchResult = [self runCommand:
                             @"team::members",
                             @"fetchGlobalIDs", [NSNumber numberWithBool:YES],
                             @"team", _globaID(self->team), nil];

        searchResult = [self runCommand:@"person::get-by-globalid",
                             @"gids", searchResult,
                             @"groupBy", @"globalID",                   
                             @"attributes", [NSArray arrayWithObjects:@"name",
                                              @"firstname", @"login", nil],
                             nil];

        enumerator = [searchResult objectEnumerator];

        while ((obj = [enumerator nextObject])) {
          obj = [obj mutableCopy];
          AUTORELEASE(obj);
          [obj setObject:[NSMutableDictionary dictionary]
               forKey:@"accessRights"];
          [self _addObj:obj toCompanies:self->companies];
        }
        {
          NSArray *sA; 

          sA = [self->companies sortedArrayUsingKeyOrderArray:
                           [NSArray arrayWithObjects:
                             [EOSortOrdering sortOrderingWithKey:@"login"
                               selector:EOCompareAscending],
                             [EOSortOrdering sortOrderingWithKey:@"name"
                               selector:EOCompareAscending],
                             [EOSortOrdering sortOrderingWithKey:@"firstname"
                               selector:EOCompareAscending],
                                              nil]];
          self->companies = [sA mutableCopy];
        }
      }
    }
  }
}

- (void)appendToResponse:(WOResponse *)_response inContext:(WOContext *)_ctx {
  [self syncVars];
  [super appendToResponse:_response inContext:_ctx];
}

- (void)takeValuesFromRequest:(WORequest *)_request inContext:(WOContext *)_ctx {
  [super takeValuesFromRequest:_request inContext:_ctx];
  [self syncVars];
}

- (void)syncSleep {
  self->syncState = NO;
  RELEASE(self->searchString); self->searchString = nil;
  [super syncSleep];
}
/* actions */

- (id)search {
  return nil;
}

/* /actions */

- (BOOL)hasAccountSelection {
  return [self->companies count];
}

- (BOOL)isTeam {
  return [[self->item valueForKey:@"isTeam"] boolValue];
}

/* accessors */

- (id)companies {
  return self->companies;
}
- (void)setCompanies:(id)_com {
  ASSIGN(self->companies, _com);
}

- (id)team {
  return self->team;
}
- (void)setTeam:(id)_team {
  ASSIGN(self->team, _team);
}

- (BOOL)isViewerMode {
  return self->isViewerMode;
}
- (void)setIsViewerMode:(BOOL)_b {
  self->isViewerMode = _b;
}

- (id)item {
  return self->item;
}
- (void)setItem:(id)_item {
  ASSIGN(self->item, _item);
}
- (id)accessItem {
  return self->accessItem;
}
- (NSString *)accessImageName {
  if (self->isViewerMode)
    return [NSString stringWithFormat:@"icon_permissions_viewer_%@.gif",
                     self->accessItem];

  return [NSString stringWithFormat:@"icon_permissions_%@.gif",
                   self->accessItem];
}
- (NSString *)accessAlternateText {
  return [[self labels] valueForKey:[NSString stringWithFormat:@"permissions_%@",
                                              self->accessItem]];
}

- (void)setAccessItem:(id)_accessItem {
  ASSIGN(self->accessItem, _accessItem);
}

- (BOOL)currentAccessRight {
  return [[[self->item valueForKey:@"accessRights"]
                       valueForKey:self->accessItem] boolValue];
}
- (void)setCurrentAccessRight:(BOOL)_right {
  [[self->item valueForKey:@"accessRights"]
               takeValue:[NSNumber numberWithBool:_right]
               forKey:self->accessItem];
}
- (int)accessListCount {
  return [self->accessChecks count] + 1;
}
- (NSString *)searchString {
  return self->searchString;
}
- (void)setSearchString:(NSString *)_str {
  ASSIGN(self->searchString, _str);
}

- (void)setAccessList:(NSMutableDictionary *)_accessList {
  if ([_accessList isNotNull]) {
    NSAssert1([_accessList isKindOfClass:[NSMutableDictionary class]],
              @"accessList should be a mutable Dicionary <%@>", _accessList);
    ASSIGN(self->accessList, _accessList);
  }
  else {
    RELEASE(self->accessList); self->accessList = nil;
  }
}

- (NSArray *)accessChecks {
  return self->accessChecks;
}
- (void)setAccessChecks:(NSArray *)_a {
  ASSIGN(self->accessChecks, _a);
}

- (id)accessList {
  return self->accessList;
}

- (BOOL)isInTable {
  return self->isInTable;
}
- (void)setIsInTable:(BOOL)_b {
  self->isInTable = _b;
}

- (id)labels {
  return self->myLabels;
}
- (void)setLabels:(id)_id {
  ASSIGN(self->myLabels, _id);
}


  /* /accessors */

@end /* SkyAccessList */



