/*
  Copyright (C) 2000-2003 SKYRIX Software AG

  This file is part of OGo

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/
// $Id: SkyCalendarScript.m,v 1.2 2003/07/11 16:45:57 jan Exp $

#include <OGoFoundation/LSWComponent.h>

@interface SkyCalendarScript : LSWComponent
{
  NSString *scriptPath; // > optional: scriptPath, default: skycalendar.js
}
@end /* SkyCalendarScript */

#include "common.h"

@implementation SkyCalendarScript

- (id)init {
  if ((self = [super init])) {
    self->scriptPath = nil;
  }
  return self;
}

#if !LIB_FOUNDATION_BOEHM_GC
- (void)dealloc {
  RELEASE(self->scriptPath);
  [super dealloc];
}
#endif

// accessors
- (void)setScriptPath:(NSString *)_path {
  ASSIGN(self->scriptPath,_path);
}
- (NSString *)scriptPath {
  return self->scriptPath;
}

- (NSString *)scriptURL {
  WOResourceManager *rm;
  NSString *url;

  WOSession *s;

  if ([self->scriptPath length]) return scriptPath;

  s = [self session];
  
  rm = [[WOApplication application] resourceManager];
  
  url = [rm urlForResourceNamed:@"skycalendar.js"
            inFramework:nil
            languages:[s languages]
            request:[[self context] request]];
  
  if (url == nil) {
    NSLog(@"WARNING[%s]: couldn't locate calendar script",
          __PRETTY_FUNCTION__);
    url = @"/Skyrix.woa/WebServerResources/skycalendar.js";
  }

  return url;
}

- (void)appendToResponse:(WOResponse *)_response inContext:(WOContext *)_ctx {
  if (![[_ctx valueForKey:@"SkyCalendarScriptIncluded"] boolValue]) {
    [super appendToResponse:_response inContext:_ctx];
    [_ctx takeValue:[NSNumber numberWithBool:YES]
          forKey:@"SkyCalendarScriptIncluded"];
  }
}

@end /* SkyCalendarScript */
