/*
  Copyright (C) 2000-2003 SKYRIX Software AG

  This file is part of OGo

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/
// $Id: SkyBusinessCardGathering.m,v 1.3 2003/08/07 18:38:34 martin Exp $

#import <EOControl/EOControl.h>
#import "common.h"
#import "SkyBusinessCardGathering.h"
#include <OGoContacts/SkyEnterpriseDataSource.h>
#include <OGoContacts/SkyPersonDataSource.h>
#include <OGoContacts/SkyPersonDocument.h>

@implementation SkyBusinessCardGathering

- (id)init {
  if ((self = [super init])) {
    NSZone *z = [self zone];
    self->gatheringPerson   = [[NSMutableDictionary alloc]
                                                    initWithCapacity:16];
    self->gatheringCompany  = [[NSMutableDictionary alloc]
                                                    initWithCapacity:16];
    self->companySearchList = [[NSMutableArray alloc]
                                               initWithCapacity:64];
    self->addedCompanies    = [[NSMutableArray alloc]
                                               initWithCapacity:64];
    self->otherPhones       = [[NSMutableArray alloc]
                                               initWithCapacity:5];
    self->categories    = nil;
    self->categoryIndex = 0;
    {
      NSEnumerator   *p     = nil;
      NSMutableArray *array = nil;
      id             obj    = nil;
      NSArray        *wanted = nil;

      p = [[[[(LSWSession *)[self session] userDefaults]
                    dictionaryForKey:@"LSTeleType"]
                    objectForKey:@"Person"] objectEnumerator];
      wanted = [[(LSWSession *)[self session] userDefaults]
                             arrayForKey:@"person_gathering_phones"];
      array = [[NSMutableArray alloc] initWithCapacity:16];

      while ((obj = [p nextObject])) {
        if ([wanted containsObject:obj]) {
          [array addObject:[NSMutableDictionary dictionaryWithObject:obj
                                                forKey:@"type"]];
        }
        else {
          [self->otherPhones addObject:
               [NSMutableDictionary dictionaryWithObject:obj forKey:@"type"]];
        }
      }
      self->phones = [array copyWithZone:z];
      RELEASE(array);
    }
  }
  return self;
}

- (void)dealloc {
  [self->gatheringPerson release];
  [self->gatheringCompany release];
  [self->item release];
  [self->companySearchList release];
  [self->addedCompanies release];
  [self->searchCompanyField release];
  [self->phones release];
  [self->otherPhones release];
  [self->categories release];
  [super dealloc];
}

- (NSMutableDictionary *)gatheringPerson {
  return self->gatheringPerson;
}

- (NSMutableDictionary *)gatheringCompany {
  return self->gatheringCompany;
}

- (void)setCompanySearchList:(NSMutableArray *)_list {
  ASSIGN(self->companySearchList, _list);
}
- (NSMutableArray *)companySearchList {
  return self->companySearchList;
}

- (void)setAddedCompanies:(NSMutableArray *)_comp {
  ASSIGN(self->addedCompanies, _comp);
}
- (NSMutableArray *)addedCompanies {
  return self->addedCompanies;
}

- (void)setSearchCompanyField:(NSString *)_field {
  ASSIGN(self->searchCompanyField, _field);
}
- (NSString *)searchCompanyField {
  return self->searchCompanyField;
}

- (void)setItem:(id)_item {
  ASSIGN(self->item, _item);
}
- (id)item {
  return self->item;
}

- (NSArray *)phones {
  return self->phones;
}

- (void)setCategoryIndex:(int)_idx {
  self->categoryIndex = _idx;
}
- (int)categoryIndex {
  return self->categoryIndex;
}

static NSString *nilDummy = nil;
- (NSString *)nilDummy {
  if (nilDummy == nil)
    nilDummy = [[NSString alloc] init];
  return nilDummy;
}

- (void)_setCategoryCount:(int)_cnt {
  [[[self session] userDefaults]
          setObject:[NSNumber numberWithInt:_cnt]
          forKey:@"person_gathering_category_count"];
}
- (int)categoryCount {
  unsigned cnt =
    [[[[self session] userDefaults]
             objectForKey:@"person_gathering_category_count"] intValue];
  if (cnt < 1) {
    [self _setCategoryCount:1];
    return 1;
  }
  return cnt;
}

- (NSMutableArray *)categories {
  int cnt = [self categoryCount];
  if (self->categories == nil) {
    self->categories =
      [[NSMutableArray alloc] initWithCapacity:cnt];
  }
  while ([self->categories count] < cnt)
    [self->categories addObject:[self nilDummy]];
  while ([self->categories count] > cnt) {
    [self->categories removeLastObject];
  }
  return self->categories;
}
- (void)setCategory:(NSString *)_cat {
  NSMutableArray *ma = [self categories];
  if (self->categoryIndex >= [self categoryCount]) {
    NSLog(@"WARNING[%s]: invalid category index: %d [count: %d]",
          self->categoryIndex, [self categoryCount]);
    return;
  }
  [ma replaceObjectAtIndex:self->categoryIndex
      withObject:(_cat == nil) ? [self nilDummy] : _cat];
}
- (NSString *)category {
  if ([[self categories] count] < (self->categoryIndex+1)) return nil;
  {
    NSString *cat = [[self categories] objectAtIndex:self->categoryIndex];
    return (cat == [self nilDummy]) ? nil : cat;
  }
}

- (BOOL)isLastCategory {
  return ((self->categoryIndex + 1) == [self categoryCount]);
}
- (BOOL)moreThan1Category {
  return ([self categoryCount] > 1);
}
- (BOOL)hasMoreCategories {
  return ([[[self session] categoryNames] count] > [self categoryCount]);
}

/* actions for the categories */
- (id)decreaseCategories {
  unsigned cnt = [self categoryCount];
  if (cnt > 1) [self _setCategoryCount:cnt-1];
  [[[self session] userDefaults] synchronize];
  return nil;
}
- (id)increaseCategories {
  [self _setCategoryCount:[self categoryCount] + 1];
  [[[self session] userDefaults] synchronize];
  return nil;
}

- (NSString *)phoneLabel {
  return [[self labels] valueForKey:[self->item objectForKey:@"type"]];
}

- (void)setPhoneInfo:(NSString *)_info {
  [self->item takeValue:_info forKey:@"info"];
}
- (NSString *)phoneInfo {
  return [self->item valueForKey:@"info"];
}
- (void)setPhoneNumber:(NSString *)_number {
  [self->item takeValue:_number forKey:@"number"];
}
- (NSString *)phoneNumber {
  return [self->item valueForKey:@"number"];
}

- (BOOL)isEditorPage {
  return YES;
}

- (id)companySearch {
  NSArray *result = nil;

  [self->companySearchList removeAllObjects];
  
  result = [self runCommand:
                   @"enterprise::extended-search",
                   @"operator",    @"OR",
                   @"description", self->searchCompanyField,
                   nil];
  
  [self->companySearchList addObjectsFromArray:(result == nil)
       ? [NSArray array]
       : result];
  return nil;
}

- (id)save {
  NSNumber *ownerId = nil;
  id       person   = nil;
  NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];

  if ([self->gatheringPerson valueForKey:@"name"] == nil) {
    [self setErrorString:@"no name for person is set"];
    return nil;
  }
  if (([self->gatheringCompany valueForKey:@"description"] == nil) &&
      ([self->addedCompanies count] == 0)) {
    [self setErrorString:@"no name for enterprise is set"];
    return nil;
  }

  [self->gatheringPerson
       setObject:[self->gatheringPerson objectForKey:@"nickname"]
       forKey:@"description"];
  
  ownerId = [[[self session] activeAccount] valueForKey:@"companyId"];    
  {
    NSArray        *t;
    NSMutableArray *tels;

    tels = [NSMutableArray arrayWithArray:self->otherPhones];
    [tels addObjectsFromArray:self->phones];

    t= [tels sortedArrayUsingKeyOrderArray:
             [NSArray arrayWithObject:
                      [EOSortOrdering sortOrderingWithKey:@"type"
                                      selector:EOCompareAscending]]];

    [self->gatheringPerson setObject:t forKey:@"telephones"];
  }

  [self->gatheringPerson takeValue:ownerId forKey:@"ownerId"];
  {
    NSMutableString *str = [NSMutableString stringWithCapacity:128];
    NSEnumerator    *e   = [self->categories objectEnumerator];
    NSString        *one = nil;
    int             idx  = -1;

    while ((one = [e nextObject])) {
      idx++;
      if (one == [self nilDummy]) continue;
      if ([self->categories indexOfObject:one] == idx) {
        // no double categories
        if ([str length] == 0) [str appendString:one];
        else                   [str appendFormat:@", %@", one];
      }
    }

    [self->gatheringPerson setObject:str forKey:@"keywords"];
  }
  person = [self runCommand:@"person::new" arguments:self->gatheringPerson];
  //[nc postNotificationName:SkyNewPersonNotification object:nil];
  [nc postNotificationName:SkyNewPersonNotification object:person];

  if ([self->addedCompanies count] > 0) {
    NSEnumerator   *enumerator;
    NSMutableArray *persons    = nil;
    id             obj         = nil;

    enumerator  = [self->addedCompanies objectEnumerator];
    while ((obj = [enumerator nextObject])) {
      RELEASE(persons); persons = nil;
      persons = [[self runCommand:@"enterprise::get-persons",
                       @"enterprise", obj, nil] mutableCopyWithZone:NULL];

      if (![persons containsObject:person]) {
        [persons addObject:person];
        [self runCommand:@"enterprise::set-persons",
              @"group",   obj,
              @"members", persons, nil];        
        [self runCommand:@"enterprise::get-persons", @"enterprise", obj, nil];
      }
    }
    RELEASE(persons); persons = nil;
  }
  else {
    id company = nil;
    [self->gatheringCompany
         setObject:[self->gatheringPerson objectForKey:@"comment"]
         forKey:@"comment"];
    [self->gatheringCompany
         setObject:[self->gatheringPerson objectForKey:@"keywords"]
         forKey:@"keywords"];
    [self->gatheringCompany
         setObject:[NSArray arrayWithObject:person]
         forKey:@"persons"];
    [self->gatheringCompany setObject:ownerId forKey:@"ownerId"];
    [self->gatheringCompany setObject:[EONull null] forKey:@"contactId"];    

    company =
      [self runCommand:@"enterprise::new" arguments:self->gatheringCompany];
    [nc postNotificationName:SkyNewEnterpriseNotification object:nil];

    {
      NSEnumerator *enumerator = nil;
      id           obj         = nil;

      enumerator = [[company valueForKey:@"toAddress"] objectEnumerator];

      while ((obj = [enumerator nextObject])) {
        if ([[obj valueForKey:@"type"] isEqual:@"bill"]) {
          [self->gatheringCompany setObject:[obj valueForKey:@"addressId"]
               forKey:@"addressId"];
        }
      }
    }
    
    [self->gatheringCompany setObject:@"bill" forKey:@"type"];
    [self->gatheringCompany setObject:[company valueForKey:@"companyId"]
         forKey:@"companyId"];

    if ([self->gatheringCompany valueForKey:@"addressId"] == nil) {
      [self runCommand:@"address::new" arguments:self->gatheringCompany];
    }
    else {
      [self runCommand:@"address::set" arguments:self->gatheringCompany];
    }
  }
  [self leavePage];
  return nil;;
}

- (id)cancel {
  [self leavePage];
  return nil;
}

// kvc

- (void)presetGatheringPerson:(id)_person {
  NSDictionary *values = nil;
  if ([_person isKindOfClass:[SkyPersonDocument class]]) {
    values = [_person asDict];
  }
  else if ([_person isKindOfClass:[NSDictionary class]])
    values = _person;

  if (values != nil)
    [self->gatheringPerson addEntriesFromDictionary:values];
  else {
    NSLog(@"%s: unable to preset values from object: %@ (%@)",
          __PRETTY_FUNCTION__, _person, NSStringFromClass([_person class]));
    [self setErrorString:@"unable to preset values"];
  }
}

- (void)takeValue:(id)_val forKey:(id)_key {
  if ([_key isEqualToString:@"presetGatheringPerson"]) {
    [self presetGatheringPerson:_val];
    return;
  }
  [super takeValue:_val forKey:_key];
}

@end
