/*
  Copyright (C) 2000-2003 SKYRIX Software AG

  This file is part of OGo

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/
// $Id: SkyImapDownloadAction.m,v 1.3 2003/10/27 17:40:57 jan Exp $

#import <NGObjWeb/WODirectAction.h>

@interface SkyImapDownloadAction : WODirectAction
{
}
@end /* SkyImapDownloadAction */

#include <OGoFoundation/OGoFoundation.h>
#include "common.h"

@interface NSObject(Private)
- (NGImap4Context *)imapContext;
+ (NGImap4Context *)sessionImapContext:(id)_ses;
@end /* NSObject(Private) */

@implementation SkyImapDownloadAction 

- (id<WOActionResults>)missingSession:(NSString *)_action {
  return nil;
}

- (id<WOActionResults>)_downloadInline:(BOOL)_inline {
  WOSession *sn;
  NSURL     *url;
  NGImap4Context *imapCtx;
  NGImap4Folder  *folder;
  NSData         *data;
  WORequest      *req;
  NGMimeType     *type;
  NSString       *encoding, *contentDisp, *filename, *path, *tmp;
  
  filename = nil;
  req      = [self request];
  type     = [NGMimeType mimeType:[req formValueForKey:@"mimeType"]];

  if ([[NSUserDefaults standardUserDefaults]
                       boolForKey:@"UseOnly7BitHeadersForMailBlobDownload"]) {
    BOOL                changed;

    changed  = NO;
    filename = [req formValueForKey:@"filename"];
    if (![filename length])
      filename = [type valueOfParameter:@"name"];


    if ([filename length]) {
      unsigned const char *cstr;
      unsigned char       *buf;
      int                 i, clen;\

      cstr = [filename cString];
      clen = [filename cStringLength];
      buf = alloca(clen + 1);
      for (i = 0; i < clen; i++) {
        if (isalnum(cstr[i]) || cstr[i] == '.')
          buf[i] = cstr[i];
        else {
          changed = YES;
          buf[i]  = '_';
        }
      }
      if (changed) {
        buf[clen] = '\0';
        filename = [NSString stringWithCString:buf length:clen];
      }
    }
    if (changed) {
      type = [NGMimeType mimeType:[type type]
                         subType:[type subType]
                         parameters:[NSDictionary dictionaryWithObject:filename
                                                  forKey:@"name"]];
    }
  }
  encoding = [req formValueForKey:@"encoding"];
  
  if ((sn  = [self existingSession]) == nil) {
    [self logWithFormat:@"no session active !"];
    return [self missingSession:@"get"];
  }

  tmp = [req formValueForKey:@"url"];

  if ([tmp length]) {
    imapCtx = [NSClassFromString(@"SkyImapContextHandler")
                                sessionImapContext:[self session]];
    url     = [NSURL URLWithString:[req formValueForKey:@"url"]];
    path    = [url path];

    if (!(folder = [imapCtx folderWithName:
                            [path stringByDeletingLastPathComponent]])) {
      NSLog(@"%s: couldn`t find folder at path %@", __PRETTY_FUNCTION__,
            [path stringByDeletingLastPathComponent]);
      return nil;
    }
    if ([folder isKindOfClass:[NGImap4ServerRoot class]]) {
      NSLog(@"%s: couldn`t load folder for url %@",
            __PRETTY_FUNCTION__, url);
      return nil;
    }
    data = [folder blobForUid:[[path lastPathComponent] intValue]
                   part:[[[url query] componentsSeparatedByString:@"="]
                               lastObject]];
    if (!data) {
      NSLog(@"%s: couldn`t fetch blob for folder %@ path %@ uid %d part %@",
            __PRETTY_FUNCTION__, folder, path, [[path lastPathComponent]
                                                      intValue],
            [[[url query] componentsSeparatedByString:@"="] lastObject]);
      return nil;
    }
  }
  else {
    tmp = [req formValueForKey:@"data_key"];

    data = [[[[self session] valueForKey:tmp] retain] autorelease];
    [[self session] removeObjectForKey:tmp];
  }
  if ([encoding length]) {
    NGMimePartParser *parser;
    id tmp;

    parser = [[NGMimePartParser alloc] init];

    tmp = [parser applyTransferEncoding:[encoding lowercaseString]
                  onData:data];
    if (tmp)
      data = tmp;
    else {
      NSLog(@"%s: encoding failed for %@", __PRETTY_FUNCTION__, encoding);
    }
    [parser release]; parser = nil;
  }
#if 0  
  if ([[encoding lowercaseString] isEqualToString:@"base64"]) {
    id tmp;

    tmp = [data dataByDecodingBase64];
    if (tmp)
      data = tmp;
  }
#endif
  if (!_inline) {
    if (![filename length])
      filename = [req formValueForKey:@"filename"];

    if (![filename length])
      filename = [type valueOfParameter:@"name"];

    if (![filename length])
      filename = @"download";
    
    contentDisp = [NSString stringWithFormat:@"attachment;filename=\"%@\"",
                            filename];
  }
  else {
    contentDisp = nil;
  }
  return [LSWMimeContent mimeContent:data ofType:type
                         contentDisposition:contentDisp
                         inContext:[sn context]];
}
  

- (id<WOActionResults>)downloadAction {
  return [self _downloadInline:NO];
}

- (id<WOActionResults>)getAction {
  return [self _downloadInline:YES];
}

@end /* SkyImapDownloadAction */

