/*
  Copyright (C) 2000-2003 SKYRIX Software AG

  This file is part of OGo

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/
// $Id: SkyImapMailPrintViewer.m,v 1.3 2004/01/20 02:10:08 helge Exp $

#include <OGoFoundation/LSWComponent.h>

@class NSArray;

@interface SkyImapMailPrintViewer : LSWComponent
{
  id      emailContent;
  id      object;
  NSArray *headers;
  id      header;
}

@end /* SkyImapMailPrintViewer */

#include "common.h"
#include <NGMail/NGMimeMessage.h>

@implementation SkyImapMailPrintViewer

- (void)dealloc {
  RELEASE(self->emailContent);
  RELEASE(self->object);
  RELEASE(self->headers);
  RELEASE(self->header);
  [super dealloc];  
}

/* accessors */

- (id)emailContent {
  return self->emailContent;
}

- (id)object {
  return self->object;
}

- (NSArray *)headers {
  if (self->headers == nil) {
    self->headers =
      [[NSArray alloc] initWithObjects:@"subject", @"from",
                       @"to", @"date", nil];
  }
  return self->headers;
}

- (void)setHeader:(id)_header {
  ASSIGN(self->header,_header);
}
- (id)header {
  return self->header;
}

- (NSString *)printTitle {
  return [@"OGo Mails: " stringByAppendingString:
	     [self->object valueForKey:@"subject"]];
}

- (id)headerLabel {
  return [[self labels] valueForKey:self->header];
}
- (id)headerValue {
  NSMutableString *ms;
  NSEnumerator    *e;
  id              one;

  ms = [NSMutableString stringWithCapacity:32];
  e  = [self->emailContent valuesOfHeaderFieldWithName:self->header];
  while ((one = [e nextObject])) {
    if ([one isKindOfClass:[NSCalendarDate class]]) {
      one = [one descriptionWithCalendarFormat:@"%Y-%m-%d %H:%M"];
    }
    else if (![one isKindOfClass:[NSString class]]) {
      one = [one description];
    }
    
    if ([ms length])
      [ms appendFormat:@", %@", one];
    else
      [ms appendString:one];
  }
  return ms;
}

/* KVC */

- (void)takeValue:(id)_val forKey:(id)_key {
  if ([_key isEqualToString:@"object"]) {
    ASSIGN(self->object,_val);
  }
  else if ([_key isEqualToString:@"emailContent"]) {
    ASSIGN(self->emailContent,_val);
  }
  else
    [super takeValue:_val forKey:_key];
}

@end /* SkyImapMailPrintViewer */
