/*
  Copyright (C) 2000-2003 SKYRIX Software AG

  This file is part of OGo

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/
// $Id: SkyVacationViewer.m,v 1.2 2003/07/11 16:45:59 jan Exp $

#include <OGoFoundation/LSWViewerPage.h>

@class NSString, NGImap4Context;

@interface SkyVacationViewer : LSWViewerPage
{
  NSDictionary *vacation;
  NSDictionary *forward;
}
@end /* SkyVacationViewer */

#include "common.h"
#include "LSWImapMails.h"
#include "LSWImapMailFilterManager.h"

@implementation SkyVacationViewer

- (void)syncAwake {
  NSEnumerator *enumerator;
  id           entry;
  
  RELEASE(self->vacation); self->vacation = nil;
  RELEASE(self->forward);  self->forward = nil;
  
  enumerator = [[LSWImapMailFilterManager vacationForUser:
                                          [[self session] activeAccount]]
                                          objectEnumerator];

  while ((entry = [enumerator nextObject])) {
    if ([[entry objectForKey:@"kind"] isEqual:@"vacation"]) {
      self->vacation = [entry mutableCopy];
    }
    else if ([[entry objectForKey:@"kind"] isEqual:@"forward"]) {
      self->forward = [entry mutableCopy];
    }
    if (self->forward && self->vacation)
      break;
  }
  [super syncAwake];
}

- (void)syncSleep {
  RELEASE(self->vacation); self->vacation = nil;
  RELEASE(self->forward);  self->forward = nil;
  [super syncSleep];
}

- (id)new {
  NGMimeType   *mt = nil;
  WOComponent  *ct = nil;

  mt = [NGMimeType mimeType:@"objc" subType:@"imap-vacation"];
  ct = [[self session] instantiateComponentForCommand:@"new" type:mt];
  return ct;
}
- (id)delete {
  return nil;
}
- (id)edit {
  NGMimeType   *mt = nil;
  WOComponent  *ct = nil;

  mt = [NGMimeType mimeType:@"objc" subType:@"imap-vacation"];
  ct = [[self session] instantiateComponentForCommand:@"edit" type:mt];
  return ct;
}
- (BOOL)hasVacation {
  return self->vacation?YES:NO;
}

- (BOOL)hasVacationElse {
  return ![self hasVacation];
}

- (NSDictionary *)vacation {
  return self->vacation;
}

- (NSDictionary *)forward {
  return self->forward;
}

- (NSString *)keepMailString {
  return [[self labels] valueForKey:
                        ([[self->forward objectForKey:@"keepMails"]
                                         boolValue])?@"YES":@"NO"];
}

@end /* SkyVacationViewer */


