/*
  Copyright (C) 2000-2003 SKYRIX Software AG

  This file is part of OGo

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/
// $Id: LSWDocumentEditor.m,v 1.2 2003/07/11 16:46:01 jan Exp $

#include <OGoFoundation/LSWEditorPage.h>

@class NSString, NSData, EOGlobalID;

@interface LSWDocumentEditor : LSWEditorPage
{
@private
  NSData   *data;
  NSString *filePath;
  id	   parentFolder;
  id       project;
  id       doc;
  BOOL     autoRelease; 
  BOOL     wasReleased;
  BOOL     isAttrEdit;
  BOOL     isImport;
  id       item;

  // file manager stuff
  id         fileManager;
  EOGlobalID *fileGID;
}

- (void)setParentFolder:(id)_folder;
- (void)setProject:(id)_project;

- (BOOL)autoRelease;
- (void)setAutoRelease:(BOOL)_autoRelease;

@end

@interface LSWDocumentEditor(SkyProjectFileManager)
- (BOOL)hasVersioning;
- (BOOL)hasLocking;
- (id)fileManager;
- (EOGlobalID *)fileId;
- (NSString *)fileName;
- (NSString *)folderPath;
- (id)_saveAndRelease:(BOOL)_release unlock:(BOOL)_unlock;
- (id)printErrorWithSource:(NSString *)_src destination:(NSString *)_dest;
@end
    


#import "common.h"

@implementation LSWDocumentEditor

- (id)init {
  if ((self = [super init])) {
    static BOOL DidInitializeBundles = NO;    

    id     ctx;
    
    self->isImport = NO;

    ctx = [(LSWSession *)[self session] commandContext];

    if (!DidInitializeBundles) {
      NGBundleManager *bm;
      NSBundle        *bundle;

      DidInitializeBundles = YES;
  
      bm     = [NGBundleManager defaultBundleManager];

      bundle = [bm bundleWithName:@"OGoFileSystemProject" type:@"ds"];
      NSLog(@"%s: load bundle %@", __PRETTY_FUNCTION__, bundle);
      [bundle load];

      bundle = [bm bundleWithName:@"OGoDatabaseProject" type:@"ds"];
      NSLog(@"%s: load bundle %@", __PRETTY_FUNCTION__, bundle);
      [bundle load];
    }
    
  }
  return self;
}

- (void)dealloc {
  [self->data         release];
  [self->filePath     release];
  [self->parentFolder release];
  [self->project      release];
  [self->doc          release];
  [self->item         release];

  // file manager stuff
  [self->fileManager release];
  [self->fileGID     release];
  [super dealloc];
}

- (void)sleep {
  [super sleep];
  [self setErrorString:nil];
}

- (void)clearEditor {
  [self->data);         self->data         = nil;
  RELEASE(self->filePath);     self->filePath     = nil;
  RELEASE(self->parentFolder); self->parentFolder = nil;
  RELEASE(self->project);      self->project      = nil;
  RELEASE(self->doc);          self->doc          = nil;
  [super clearEditor];
}

- (BOOL)prepareForEditCommand:(NSString *)_command
  type:(NGMimeType *)_type
  configuration:(NSDictionary *)_cmdCfg
{
  id obj = [self object];
  id pj  = [obj valueForKey:@"project"];

  if (pj != nil) {
    ASSIGN(self->project, pj);
  }
  
  self->wasReleased = [[obj valueForKey:@"wasReleased"] boolValue];
  self->isAttrEdit  = [[obj valueForKey:@"attrEdit"] boolValue];

  [obj removeObjectForKey:@"wasReleased"];
  {
    id d = [obj valueForKey:@"toDoc"];
    ASSIGN(self->doc, d);
  }

  if ([[self->doc valueForKey:@"isIndexDoc"] boolValue]) {
    self->autoRelease = YES;
  }

  return YES;
}

// accessors

- (void)setAutoRelease:(BOOL)_bool {
  self->autoRelease = _bool;
}
- (BOOL)autoRelease {
  return self->autoRelease;
}

- (void)setIsAttrEdit:(BOOL)_bool {
  self->isAttrEdit  = _bool;
}
- (BOOL)isAttrEdit  {
  return self->isAttrEdit;
}

- (void)setIsImport:(BOOL)_bool {
  self->isImport = _bool;
}
- (BOOL)isImport    {
  return self->isImport;
}

- (void)setParentFolder:(id)_id {
  ASSIGN(self->parentFolder, _id);
}
- (id)parentFolder {
  return self->parentFolder;
}

- (void)setProject:(id)_id {
  ASSIGN(self->project, _id);
}
- (id)project      {
  return self->project;
}

- (void)setData:(NSData *)_data {
  ASSIGN(self->data, _data);
}
- (NSData *)data {
  return self->data;
}

- (void)setFilePath:(NSString *)_path {
  ASSIGN(self->filePath, _path);
}
- (NSString *)filePath {
  return self->filePath;
}

- (void)setItem:(id)_id {
  ASSIGN(self->item, _id);
}
- (id)item {
  return self->item;
}

- (id)document {
  return [self snapshot];
}

- (id)selectedProject {
  return [[self snapshot] valueForKey:@"project"];
}

- (void)setSelectedProject:(id)_id {
  [[self snapshot] takeValue:_id forKey:@"project"];
}

- (NSString *)windowTitle {
  return (self->isAttrEdit)
    ? [[self labels] valueForKey:@"windowTitleAttributeDocumentEditor"]
    : [[self labels] valueForKey:@"windowTitleDocumentEditor"];
}

- (BOOL)isDeleteDisabled {
  if (![self isInNewMode] &&
      ![[self->doc valueForKey:@"isIndexDoc"] boolValue]) {
    BOOL isEnabled = NO;
    id   myAccount = [[self session] activeAccount];
    id   accountId = [myAccount valueForKey:@"companyId"];
    id   obj       = self->doc;  
  
    isEnabled = (([accountId isEqual:[obj valueForKey:@"firstOwnerId"]]) ||
                 ([accountId intValue] == 10000));

    return !isEnabled;
  }
  return YES;
}

- (BOOL)isContactAttrEnabled {
  return [[[self session] userDefaults]
                 boolForKey:@"SkyEnableContactAttrInDocuments"];
}

- (NSString *)insertNotificationName {
  return LSWNewDocumentNotificationName;
}
- (NSString *)updateNotificationName {
  return LSWUpdatedDocumentNotificationName;
}
- (NSString *)deleteNotificationName {
  return LSWDeletedDocumentNotificationName;
}

//actions

- (BOOL)checkConstraints {
  NSMutableString *error;
  
  error  = [NSMutableString stringWithCapacity:128];

  if ([self isImport]) {
    if (self->project == nil)
      [error appendString:[[self labels] valueForKey:@"error_no_project"]];
  }
  if ((self->data == nil ||
       (([self->data length] == 0) && (self->isImport != YES))) &&
      !self->isAttrEdit)
    [error appendString:@"File for upload is required!"];
  
  if ([error length] > 0) {
    [self setErrorString:error];
    return YES;
  }
  else {
    [self setErrorString:nil];
    return NO;
  }
}

- (BOOL)checkConstraintsForSave {
  return ![self checkConstraints];
}

- (NSString *)_fNameForPath:(NSString *)_path {
  NSArray  *cmpts    = [_path componentsSeparatedByString:@"\\"];

  if ([cmpts count] == 1) {
    cmpts = [_path componentsSeparatedByString:@"/"];
  }

  return [cmpts lastObject];
}

- (id)saveAndMove {
  if ([self checkConstraintsForSave]) {
    if ([self isInNewMode]) {
      if ([self fileManager] != nil) {
        LSWContentPage *page    = [self pageWithName:@"SkyProject4MovePanel"];
        NSDictionary   *theDoc  = nil;
        NSString       *fname   = [self fileName];
        NSString       *subject = [[self snapshot] valueForKey:@"abstract"];

        if (fname == nil)
          fname = @"";

        if (subject == nil)
          subject = @"";

        theDoc = [NSDictionary dictionaryWithObjectsAndKeys:
                     subject, @"NSFileSubject",
                     fname,   @"NSFileName",
                     [NSNumber numberWithInt:[[self data] length]], NSFileSize,
                     [self data], @"content", nil];

        [page takeValue:[NSArray arrayWithObject:theDoc] forKey:@"newDocuments"];
        [page takeValue:[self fileManager] forKey:@"fileManager"];

        [self enterPage:page];
        return page;
      }
      else {
        [super save];
        return [self activateObject:[self object] withVerb:@"move"];
      }
    }
    else { // is not in newMode
      // do nothing
    }
  }
  return nil;
}

- (id)saveAndGoBackWithCount:(int)_backCount {
  return nil;
}

- (id)insertObject {
  id       document = [self snapshot];
  NSNumber *aRel    = [NSNumber numberWithBool:self->autoRelease];
  NSString *fName   = [document valueForKey:@"title"];

  if (![self checkConstraintsForSave]) return nil;

  if ([self fileManager]) {
    [self _saveAndRelease:self->autoRelease unlock:YES];
    return nil;
  }
  else {
    if (self->parentFolder == nil) {
      [project run:@"project::get-root-document",
               @"relationKey", @"rootDocument", nil];

      self->parentFolder = [project valueForKey:@"rootDocument"];
      RETAIN(self->parentFolder);
    }

    [document takeValue:self->parentFolder           forKey:@"folder"];
    [document takeValue:self->project                forKey:@"project"];
    [document takeValue:[NSNumber numberWithBool:NO] forKey:@"isFolder"];
    [document takeValue:aRel                         forKey:@"autoRelease"];

    if (self->data != nil && self->filePath != nil) {    
      NSNumber *len = [NSNumber numberWithInt:[self->data length]];

      [document takeValue:self->data     forKey:@"data"];
      [document takeValue:self->filePath forKey:@"filePath"];
      [document takeValue:len            forKey:@"fileSize"];

      if (![fName isNotNull] || [fName length] == 0) {
        fName = [self _fNameForPath:self->filePath];
        [document takeValue:[fName stringByDeletingPathExtension] forKey:@"title"];
      }
    }
    return [self runCommand:@"doc::new" arguments:document];
  }
}

- (id)updateObject {
  NSMutableDictionary *document = nil;
  NSString            *fName    = nil;

  document = [self snapshot];
  fName    = [document valueForKey:@"title"];
  
  [document setObject:[NSNumber numberWithBool:self->autoRelease]
            forKey:@"autoRelease"];
  [document setObject:[NSNumber numberWithBool:self->isAttrEdit]
            forKey:@"linkMode"];

  if (self->data != nil) {
    NSNumber *len = [NSNumber numberWithInt:[self->data length]];

    [document setObject:self->data     forKey:@"data"];
    [document setObject:self->filePath forKey:@"filePath"];
    [document setObject:len            forKey:@"fileSize"];
    
    if (![fName isNotNull] || [fName length] == 0) {
      fName = [self _fNameForPath:self->filePath];
      [document setObject:[fName stringByDeletingPathExtension]
                forKey:@"title"];
    }
  }
  
  if (self->project != nil)
    [document setObject:self->project forKey:@"project"];

  NSAssert([document valueForKey:@"fileType"], @"no filetype set !");
    
  return [self runCommand:@"doc::set" arguments:document];
}

- (id)reallyDelete {
  id result;

  result = [self->doc run:@"doc::delete",
                        @"reallyDelete", [NSNumber numberWithBool:YES],
                        nil];
  if (result) {
    if (![self commit]) {
      [self setErrorString:@"Couldn't commit doc::delete command (rollback) !"];
      [self rollback];
      return nil;
    }
    
    [self postChange:LSWDeletedDocumentNotificationName onObject:result];
    [self back];
    [self back];
  }
  [self setErrorString:@"Couldn't delete doc (rollback) !"];
  [self rollback];
  return nil;
}

- (id)cancel {
  if([self checkConstraintsForCancel]) {
    if(self->wasReleased) {
      id result;

      result = [self runCommand:@"doc::reject" object:self->doc];
      if (result == nil) {
        return nil;
      }
      if (![self commit]) {
        [self setErrorString:@"Couldn't commit doc::reject command !"];
        return nil;
      }
    }
    return [self back];
  }
  return nil;
}

@end

#include <OGoDatabaseProject/SkyProjectFileManager.h>

@implementation LSWDocumentEditor(SkyProjectFileManager)
- (void)setFileManager:(id)_fileManager {
  ASSIGN(self->fileManager, _fileManager);
}
- (id)fileManager {
  if (self->fileManager == nil) {
    self->fileManager =
      [[SkyFileManagerFactory fileManagerInContext:
                              [[self session] commandContext]
                              forProject:self->project] retain];
  }
  return self->fileManager;
}

- (NSString *)docPath {
  EOGlobalID *fgid;
  NSString   *path;
  
  if ((fgid = [self fileId])) {
    path = [[self fileManager] pathForGlobalID:fgid];
  }
  else if ([self folderPath]) {
    path = [self folderPath];
    path = [path stringByAppendingPathComponent:[self fileName]];
  }
  else
    path = nil;
  
  return path;  
}

- (NSString *)folderPath {
  return @"/";
}

- (EOGlobalID *)fileId {
  return self->fileGID;
}

- (NSString *)fileName {
  return self->filePath;
}


- (BOOL)hasVersioning {
  return [[self fileManager] supportsVersioningAtPath:[self docPath]];
}

- (BOOL)hasLocking {
  return [[self fileManager] supportsLockingAtPath:[self docPath]];
}

- (id)_saveAndRelease:(BOOL)_release unlock:(BOOL)_unlock {
  id       fm;
  NSString *fname;
  
  fm = [self fileManager];

  [self debugWithFormat:@"save %d bytes ..", [self->data length]];
  
  if ([self fileId]) {
    /* don't need to watch path, just store the data */
    
    fname = [fm pathForGlobalID:[self fileId]];
    /* first checkout or lock file */
    
    if ([self hasVersioning]) {
      if (![fm isFileLockedAtPath:fname]) {
        if (![fm checkoutFileAtPath:fname handler:nil]) {
          [self setErrorString:@"checkout of file failed !"];
          return nil;
        }
      }
    }
    else if ([self hasLocking]) {
      if (![fm isFileLockedAtPath:fname]) {
        if (![fm lockFileAtPath:fname handler:nil]) {
          [self setErrorString:@"locking of file failed !"];
          return nil;
        }
      }
    }

    /* now write contents */
    
    if (![fm writeContents:self->data atPath:fname]) {
      return [self printErrorWithSource:fname destination:nil];
    }
    
    /* now handle auto-release/unlock */
    
    if (_release) {
      if (![fm releaseFileAtPath:fname handler:nil])
        [self setErrorString:@"couldn't release file after writing data .."];
    }
    else if (_unlock) {
      if (![fm unlockFileAtPath:fname handler:nil])
        [self setErrorString:@"couldn't unlock file after writing data .."];
    }
  }
  else {
    /* create new file */
    NSString *folderPath;
    NSDictionary *attrs   = nil;
    NSString     *subject = nil;

    subject = [[self snapshot] objectForKey:@"abstract"];
    if ([subject isNotNull])
      attrs = [NSDictionary dictionaryWithObjectsAndKeys:
                            subject, @"NSFileSubject", nil];
    else
      attrs = nil; // [NSDictionary dictionary];
    
    fname = [self fileName];
    
    if ([fname length] == 0) {
      [self setErrorString:@"missing filename !"];
      return nil;
    }
    
    folderPath = [self folderPath];
    fname      = [folderPath stringByAppendingPathComponent:fname];

#if 1
    [self logWithFormat:@"save fm='%@' %d bytes at %@ .. attrs='%@'",
            fm,
            [self->data length],
            fname, attrs];
#endif
    
    if (![fm createFileAtPath:fname
             contents:self->data
             attributes:attrs]) {
      return [self printErrorWithSource:fname destination:nil];
    }
  }

  [(SkyProjectFileManager *)[self fileManager] flush];
  
  return [[(LSWSession *)[self session] navigation] leavePage];
}

- (NSString *)_errorDescription {
  int      errorCode = 0;
  NSString *errorKey = nil;

  errorCode = [[self fileManager] lastErrorCode];
  
  if (errorCode > 0) {
    errorKey  = [NSString stringWithFormat:@"fm_error_%d", errorCode];
    return [[self labels] valueForKey:errorKey];
  }
  
  return nil;
}

- (id)printErrorWithSource:(NSString *)_src destination:(NSString *)_dest {
  NSString *str = [self _errorDescription];

  if (_src || _dest)
    str = [str stringByAppendingString:@" ("];
  if (_src) {
    str = [str stringByAppendingFormat:@"'%@'", _src];
    if (_dest)
      str = [str stringByAppendingString:@", "];
  }
  if (_dest)
    str = [str stringByAppendingFormat:@"'%@'", _dest];
  if (_src || _dest)
    str = [str stringByAppendingString:@")"];

  [self setErrorString:str];
  
  return nil;
}
@end
