/*
  Copyright (C) 2000-2003 SKYRIX Software AG

  This file is part of OGo

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/
// $Id: SkyCompanyAccessEditor.m,v 1.3 2003/11/30 23:07:27 helge Exp $

#include <OGoFoundation/LSWContentPage.h>

@class NSArray, NSMutableDictionary;
@class EOGlobalID;
@class SkyObjectPropertyManager;

@interface SkyCompanyAccessEditor : LSWContentPage
{
  EOGlobalID          *globalID;
  NSMutableDictionary *accessIds;
  NSArray             *accessChecks;
}

@end

#include <LSFoundation/SkyAccessManager.h>
#include "common.h"

@implementation SkyCompanyAccessEditor

- (void)dealloc {
  [self->globalID  release];
  [self->accessIds release];
  [super dealloc];
}

- (void)setGlobalID:(EOGlobalID *)_gid {
  NSDictionary *ids;
  
  ASSIGN(self->globalID, _gid);
  [self->accessIds release]; self->accessIds = nil;
  
  ids = [[[(id)[self session] commandContext] accessManager]
                     allowedOperationsForObjectId:self->globalID];
  self->accessIds = [ids mutableCopy];
}
- (EOGlobalID *)globalID {
  return self->globalID;
}

- (id)accessIds {
  return self->accessIds;
}
- (void)setAccessIds:(id)_id {
  ASSIGN(self->accessIds, _id);
}

- (id)accessChecks {
  return self->accessChecks;
}
- (void)setAccessChecks:(id)_id {
  ASSIGN(self->accessChecks, _id);
}

#if 0 // hh asks: what is that?
- (id)save {
  SkyAccessManager *manager;
  NSEnumerator     *enumerator;
  id               key;

  manager    = [[[self session] commandContext] accessManager];
  enumerator = [self->accessIds keyEnumerator];

  if (![manager deleteOperationForObjectID:self->globalID accessGlobalID:nil]) {
    [self setErrorString:@"Couldn`t set access rights"];
    return nil;
  }
  while ((key = [enumerator nextObject])) {
    if (![manager insertOperation:[self->accessIds objectForKey:key]
                  onObjectID:self->globalID forAccessGlobalID:key]) {
      [self setErrorString:@"Couldn`t set access rights"];
      return nil;
    }
  }
  [[NSNotificationCenter defaultCenter]
                         postNotificationName:@"SkyAccessHasChangedNotification"
                         object:self->globalID userInfo:nil];
                          
  return [[(LSWSession *)[self session] navigation] leavePage];
}

#else

- (id)save {
  SkyAccessManager *manager;
  NSDictionary     *dict;
  NSEnumerator     *enumerator;
  id               obj;
  NSMutableSet     *gidsObj;
  NSSet            *gidsNew;

  manager = [[[self session] commandContext] accessManager];
  dict    = [manager allowedOperationsForObjectId:self->globalID];
  gidsObj = [NSMutableSet setWithArray:[dict allKeys]];
  gidsNew = [NSSet setWithArray:[self->accessIds allKeys]];

  [gidsObj minusSet:gidsNew];

  enumerator = [gidsObj objectEnumerator];

  while ((obj = [enumerator nextObject])) {
    [self->accessIds setObject:@"" forKey:obj];
  }

  if (![manager setOperations:self->accessIds onObjectID:self->globalID]) {
    [self setErrorString:@"Couldn`t set access rights"];
    return nil;
  }
  [[NSNotificationCenter defaultCenter]
                         postNotificationName:@"SkyAccessHasChangedNotification"
                         object:self->globalID userInfo:nil];
                          
  return [[(LSWSession *)[self session] navigation] leavePage];
}

#endif
- (id)cancel {
  return [[(LSWSession *)[self session] navigation] leavePage];
}

@end /* SkyObjectPropertyEditor */
