/*
  Copyright (C) 2000-2003 SKYRIX Software AG

  This file is part of OGo

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/
// $Id: SkyP4DocumentDownloadLogList.m,v 1.3 2003/10/23 13:15:14 helge Exp $

#include <OGoFoundation/OGoComponent.h>

/**
 *
 *  > document   // the SkyProjectDocument to show the logs
 *
 */

@class EODataSource;
@class SkyProjectDocument;

@interface SkyP4DocumentDownloadLogList : OGoComponent
{
  SkyProjectDocument *document; // >

  EODataSource *ds;
}

@end /* SkyP4DocumentDownloadLogList */

#include "common.h"

@implementation SkyP4DocumentDownloadLogList

- (id)init {
  if ((self = [super init])) {
    self->document = nil;
    self->ds       = nil;
  }
  return self;
}

#if !LIB_FOUNDATION_BOEHM_GC
- (void)dealloc {
  RELEASE(self->document);
  RELEASE(self->ds);
  [super dealloc];
}
#endif

- (void)setDocument:(SkyProjectDocument *)_doc {
  ASSIGN(self->document,_doc);
}
- (SkyProjectDocument *)document {
  return self->document;
}

- (void)sleep {
  [super sleep];
  [(EOCacheDataSource *)self->ds clear];
}

- (id)_commandContext {
  return [(id)[self session] commandContext];
}

- (EOFetchSpecification *)_fspec {
  EOQualifier *qual =
    [EOQualifier qualifierWithQualifierFormat:@"action='download'"];

  return [EOFetchSpecification fetchSpecificationWithEntityName:@"log"
                               qualifier:qual sortOrderings:nil];
}

- (EODataSource *)dataSource {
  if (self->ds == nil) {
    self->ds = [[SkyLogDataSource alloc] initWithContext:[self _commandContext]
                                         globalID:[self->document globalID]];
    [(SkyLogDataSource *)self->ds setFetchSpecification:[self _fspec]];
    AUTORELEASE(self->ds);

    self->ds = [[EOCacheDataSource alloc] initWithDataSource:self->ds];
  }
  return self->ds;
}

@end /* SkyP4DocumentDownloadLogList */
