/*
  Copyright (C) 2000-2003 SKYRIX Software AG

  This file is part of OGo

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/
// $Id: SkyProject4BLOBViewer.m,v 1.3 2003/10/23 13:15:14 helge Exp $

#include <OGoFoundation/OGoComponent.h>

@class EOGlobalID;

@interface SkyProject4BLOBViewer : OGoComponent
{
  id         fileManager;
  EOGlobalID *documentId;
  NSString   *mimeType;
  id         viewer;
  id         document;
}

@end

#include "SkyP4DocumentRequestHandler.h"
#include "NSData+SkyTextEditable.h"
#include <NGMime/NGMimeType.h>
#include "common.h"

@implementation SkyProject4BLOBViewer

- (void)dealloc {
  [self->viewer      release];
  [self->mimeType    release];
  [self->fileManager release];
  [self->documentId  release];
  [self->document    release];
  [super dealloc];
}

/* sleep */

- (void)sleep {
  [super sleep];
  [self->viewer release]; self->viewer = nil;
}

/* accessors */

- (void)setFileManager:(id)_fm {
  ASSIGN(self->fileManager, _fm);
}
- (id)fileManager {
  return self->fileManager;
}

- (void)setDocumentId:(EOGlobalID *)_gid {
  ASSIGNCOPY(self->documentId, _gid);
}
- (id)documentId {
  return self->documentId;
}

- (void)setMimeType:(NSString *)_type {
  ASSIGNCOPY(self->mimeType, _type);
}
- (NSString *)mimeType {
  return self->mimeType;
}

- (void)setDocument:(id)_doc {
  ASSIGN(self->document, _doc);
}
- (id)document {
  return self->document;
}

- (NSString *)documentPath {
  if (([self document]))
    return [[self document] path];

  if (([self documentId]))
    return [[self fileManager] pathForGlobalID:[self documentId]];

  return nil;
}

- (NSData *)content {
  NSString *path;
  
  if ((path = [self documentPath]) == nil) {
    [self debugWithFormat:@"missing document path .."];
    return nil;
  }

  return [[self fileManager] contentsAtPath:path];
}

- (id)_makeDocViewerForType:(NGMimeType *)_type content:(NSData *)_data {
  /* TODO: split up method */
  id v;
  
  v = [[self session]
             instantiateComponentForCommand:@"docview-inline"
             type:_type
             object:_data];
  if (v) {
    NSString *path;
    id tmp;
    
    if ((path = [self documentPath])) {
      EOGlobalID *pnum;
      
      [v takeValue:path forKey:@"filename"];
      [v takeValue:[self fileManager] forKey:@"fileManager"];
      
      pnum = [[[self fileManager]
                     fileSystemAttributesAtPath:path]
                     objectForKey:@"NSFileSystemNumber"];
      
      tmp = [[self context] p4documentURLForProjectWithGlobalID:pnum
                            path:path
                            versionTag:nil];
      if (tmp)
        [v takeValue:tmp forKey:@"uri"];
    }
    
    return v;
  }

  v = [[self session]
             instantiateComponentForCommand:@"mailview"
             type:_type
             object:_data];
  if (v)
    return v;
  
  return nil;
}

- (id)viewerComponent {
  NGMimeType *mtype;
  BOOL doShow;

  doShow = [[[self session] userDefaults] boolForKey:@"LSPluginViewerEnabled"];

  if (doShow) {
    if (self->viewer) {
      if ([self->viewer respondsToSelector:@selector(object)]) {
        if ([[self->viewer object] isEqual:[self content]])
          return self->viewer;
      }
    }
    else
      ASSIGN(self->viewer, nil);
  }
  else
    ASSIGN(self->viewer, nil);

  mtype = [NGMimeType mimeType:[self mimeType]];
  
  /* lookup document viewer */

  if (doShow) {
    self->viewer = [self _makeDocViewerForType:mtype content:[self content]];
    if (self->viewer) {
      RETAIN(self->viewer);
      [self debugWithFormat:@"use viewer: %@", self->viewer];
      return self->viewer;
    }
  }
  
  /* found no proper inline viewer */
  
  if (self->viewer == nil) {
    NSData *data;
    
    if ((data = [self content]) == nil) {
      [self debugWithFormat:@"missing content .."];
    }
    
    if ([data isSkyTextEditable]) {
      mtype = [NGMimeType mimeType:@"text/plain"];
      
      self->viewer = [self _makeDocViewerForType:mtype content:data];
      if (self->viewer) {
        RETAIN(self->viewer);
        [self debugWithFormat:@"use plain/text viewer: %@", self->viewer];
        return self->viewer;
      }
    }
    else {
      [self debugWithFormat:@"data isn't text/plain .."];
    }
  }

  if (self->viewer == nil)
    [self debugWithFormat:@"did not find viewer for type %@", mtype];
  
  return self->viewer;
}

@end /* SkyProject4BLOBViewer */
