/*
  Copyright (C) 2000-2004 SKYRIX Software AG

  This file is part of OpenGroupware.org.

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/
// $Id: SkySchedulerSelectPanel.m,v 1.5 2004/03/13 23:43:29 helge Exp $

#include <OGoFoundation/LSWComponent.h>

/*
  a component to generate a fetchSpecification for a SkyAptDataSource
  includes input fields for all needed data
  used in scheduler page. (soon)

  Parameters:
  <>  timeZone    - timeZone
  <>  year        - selected year
  <>  month       - selected month
  <>  weekStart   - start of selected week
  <>  day         - selected day
  <   fetchSpecification - fetchSpecification for SkyAptDataSource
   >  mode        - viewMode (values:dayoverview|weekoverview|
                                     monthoverview|yearoverview)
*/

@class NSTimeZone, NSCalendarDate, NSArray, NSMutableArray, NSUserDefaults;

@interface SkySchedulerSelectPanel : LSWComponent
{
@protected
  NSString *searchString;
  
  NSArray        *accounts;
  NSMutableArray *selectedAccounts;
  NSArray        *selAcCache;
  NSArray        *persons;
  NSMutableArray *selectedPersons;
  NSArray        *selPerCache;
  NSArray        *teams;
  NSMutableArray *selectedTeams;
  NSArray        *selTmCache;
  NSArray        *resources;
  NSArray        *aptTypes;
  NSArray        *selectedAptTypes;
  NSMutableArray *selectedResources;
  NSArray        *selResCache;
  NSUserDefaults *defaults;

  id       item;
  id       selectedCompany;
  id       activeAccount;
  BOOL     fetchMeToo;
  BOOL     isExtended;
  int      maxSearchCount;
  BOOL     reconfigure;
  BOOL     alreadyReconfigured;
}
- (BOOL)hasAccounts;
- (BOOL)hasPersons;
- (BOOL)hasTeams;
- (BOOL)hasResources;

- (NSString *)monthLabel;
- (NSString *)extendButtonImg;
- (NSString *)extendButtonLabel;

- (NSTimeZone *)timeZone;
- (NSCalendarDate *)weekStart;
- (id)show;
- (id)extend;

@end

#include "common.h"
#import <GDLExtensions/GDLExtensions.h>
#include <OGoFoundation/OGoFoundation.h>
#include <LSFoundation/LSCommandContext.h>
#include <OGoScheduler/SkyAppointmentQualifier.h>
#include <time.h>

@interface SkySchedulerSelectPanel(PrivateMethods)
- (void)setSelectedCompany:(id)_company;
@end

@implementation SkySchedulerSelectPanel

+ (int)version {
  return 1;
}
- (BOOL)synchronizesVariablesWithBindings {
  return NO;
}

- (id)init {
  if ((self = [super init])) {
    id             me;
    EOKeyGlobalID  *gid;
    NSUserDefaults *ud;

    self->maxSearchCount = 20;
    self->reconfigure    = YES;
      
    me = [[self session] activeAccount];
    ASSIGN(self->activeAccount, me);
      
    gid = [me valueForKey:@"globalID"];
#if DEBUG
    NSAssert1(me,  @"missing active account in session %@", [self session]);
    NSAssert1(gid, @"missing global-id in active account %@", me);
#endif
      
    me = [[self runCommand:@"person::get-by-globalID",
                  @"gids",       [NSArray arrayWithObject:gid],
                  @"attributes",
                  [NSArray arrayWithObjects:@"companyId", @"name", @"firstname",
                           @"isAccount", @"login", @"globalID", nil],
                  nil] lastObject];
    [self setSelectedCompany:me];

    ud = [self runCommand:@"userdefaults::get", @"user", me, nil];
    self->defaults = [ud retain];
  }
  return self;
}

- (void)dealloc {
  [self->activeAccount    release];
  [self->defaults         release];
  [self->searchString     release];
  [self->accounts         release];
  [self->selectedAccounts release];
  [self->selAcCache       release];
  [self->persons          release];
  [self->selectedPersons  release];
  [self->selPerCache      release];
  [self->teams            release];
  [self->selectedTeams    release];
  [self->selTmCache       release];
  [self->resources        release];
  [self->selectedResources release];
  [self->selResCache      release];
  [self->selectedAptTypes release];
  [self->aptTypes         release];

  [self->item            release];
  [self->selectedCompany release];
  [super dealloc];
}

/* accessors */

- (void)_setLabelForPerson:(id)_p {
  id p;
  id d;
  
  p = _p;
  d = nil;

  // TODO: should be a NSFormatter
  if ((d = [p valueForKey:@"name"]) == nil)
    d = [p valueForKey:@"login"];
  else {
    NSString *fd = [p valueForKey:@"firstname"];
    
    if ([fd isNotNull])
      d = [NSString stringWithFormat:@"%@, %@", d, fd];
  }
  [p takeValue:d forKey:@"participantLabel"];
}

- (void)_updatePersonList:(NSArray *)_list {
  NSEnumerator *pEnum;
  id           p;
  
  pEnum =  [_list objectEnumerator];
  while ((p = [pEnum nextObject]))
    [self _setLabelForPerson:p];
}

/* accessors */

- (void)setSearchString:(NSString *)_str {
  ASSIGNCOPY(self->searchString,_str);
}
- (NSString *)searchString {
  return self->searchString;
}

- (void)setAccounts:(NSArray *)_accounts {
  ASSIGN(self->accounts,_accounts);
}
- (NSArray *)accounts {
  return self->accounts;
}

- (void)setSelectedAccounts:(NSArray *)_accounts {
  ASSIGN(self->selectedAccounts, _accounts);
}
- (NSMutableArray *)selectedAccounts {
  if (self->selectedAccounts == nil) 
    return [NSMutableArray array];

  return selectedAccounts;
}

- (void)setPersons:(NSArray *)_persons {
  ASSIGN(self->persons, _persons);
}
- (NSArray *)persons {
  return self->persons;
}

- (void)setSelectedPersons:(NSArray *)_persons {
  ASSIGN(self->selectedPersons, _persons);
}
- (NSMutableArray *)selectedPersons {
  if (self->selectedPersons == nil) 
    return [NSMutableArray array];

  return self->selectedPersons;
}

- (void)setTeams:(NSArray *)_teams {
  ASSIGN(self->teams,_teams);
}
- (NSArray *)teams {
  return self->teams;
}

- (void)setSelectedTeams:(NSArray *)_teams {
  ASSIGN(self->selectedTeams,_teams);
}
- (NSMutableArray *)selectedTeams {
  return (self->selectedTeams == nil)
    ? [NSMutableArray array]
    : self->selectedTeams;
}

- (void)setResources:(NSArray *)_resources {
  ASSIGN(self->resources,_resources);
}
- (NSArray *)resources {
  return self->resources;
}

- (void)setSelectedResources:(NSArray *)_resources {
  ASSIGN(self->selectedResources,_resources);
}
- (NSArray *)selectedResources {
  return (self->selectedResources == nil)
    ? [NSMutableArray array]
    : self->selectedResources;
}

- (void)setItem:(id)_item {
  ASSIGN(self->item,_item);
}
- (id)item {
  return self->item;
}

- (void)setSelectedCompany:(id)_company {
  if (![_company isEqual:self->selectedCompany]) {
    ASSIGN(self->selectedCompany,_company);
    self->reconfigure = YES;
  }
}
- (id)selectedCompany {
  return self->selectedCompany;
}

- (void)setFetchMeToo:(BOOL)_flag {
  if (_flag != self->fetchMeToo) {
    self->fetchMeToo = _flag;
    self->reconfigure = YES;
  }
}
- (BOOL)fetchMeToo {
  return self->fetchMeToo;
}

- (void)setIsExtended:(BOOL)_flag {
  self->isExtended = _flag;
}
- (BOOL)isExtended {
  return self->isExtended;
}

/* appointment types */
- (NSArray *)configuredAptTypes {
  NSArray *configured = nil;
  NSArray *custom     = nil;
  configured =
    [[[self session] userDefaults]
            objectForKey:@"SkyScheduler_defaultAppointmentTypes"];
  if (configured == nil) configured = [NSArray array];
  custom =
    [[[self session] userDefaults]
            objectForKey:@"SkyScheduler_customAppointmentTypes"];
  if (custom != nil)
    configured = [configured arrayByAddingObjectsFromArray:custom];
  return configured;
}
- (NSArray *)aptTypes {
  if (self->aptTypes == nil) {
    aptTypes = [self configuredAptTypes];
    RETAIN(aptTypes);
  }
  return aptTypes;
}
- (NSString *)aptTypeLabel {
  NSString *label;

  if ((label = [self->item valueForKey:@"label"]))
    return label;
  
  // TODO: should be a formatter?
  label = [[self->item valueForKey:@"type"] stringValue];
  label = [@"aptType_" stringByAppendingString:label];
  return [[self labels] valueForKey:label];
}

// single mode
- (void)setSelectedAptType:(id)_type {
  NSString *key;
  
  key = [_type valueForKey:@"type"];
  if ([key isEqual:[self->selectedAptTypes lastObject]])
    return;

  [self->selectedAptTypes release];
  if ((key == nil) || ([key isEqualToString:@"none"])) 
    self->selectedAptTypes = nil;
  else
    self->selectedAptTypes = [[NSArray alloc] initWithObjects:key, nil];
  self->reconfigure = YES;
}
- (id)selectedAptType {
  NSEnumerator *e;
  id           one;
  NSString     *wanted;
  
  e      = [[self aptTypes] objectEnumerator];
  wanted = [self->selectedAptTypes lastObject];
  
  while ((one = [e nextObject])) {
    NSString *key;
    
    key = [one valueForKey:@"type"];
    if ((![wanted length]) && [key isEqualToString:@"none"])
      return one;
    if ([wanted isEqualToString:key])
      return one;
  }
  return nil;
}

/* binding accessors */

- (void)setRealTimeZone:(NSTimeZone *)_tz {
  if ([_tz isEqual:[self timeZone]])
    return;

  [self setValue:_tz forBinding:@"timeZone"];
  self->reconfigure = YES;
}
- (NSTimeZone *)timeZone {
  return [self valueForBinding:@"timeZone"];
}
- (NSTimeZone *)realTimeZone {
  return [self timeZone];
}

- (void)setYear:(int)_y {
  [self setValue:[NSNumber numberWithInt:_y] forBinding:@"year"];
}
- (int)year {
  return [[self valueForBinding:@"year"] intValue];
}

- (void)setIsResCategorySelected:(BOOL)_flag {
  [self setValue:[NSNumber numberWithBool:_flag]
        forBinding:@"isResCategorySelected"];
}
- (BOOL)isResCategorySelected {
  return [[self valueForBinding:@"isResCategorySelected"] boolValue];
}

- (void)setMonth:(int)_m {
  [self setValue:[NSNumber numberWithInt:_m] forBinding:@"month"];
}
- (int)month {
  return [[self valueForBinding:@"month"] intValue];
}

- (void)setWeekStart:(NSCalendarDate *)_ws {
  if (![_ws isEqualToDate:[self weekStart]]) {
    [self setValue:_ws forBinding:@"weekStart"];
    [self setMonth:[_ws monthOfYear]];
    self->reconfigure = YES;
  }
}
- (NSCalendarDate *)weekStart {
  return [self valueForBinding:@"weekStart"];
}

- (void)setDay:(NSCalendarDate *)_day {
  [self setValue:_day forBinding:@"day"];
}
- (NSCalendarDate *)day {
  return [self valueForBinding:@"day"];
}

- (NSString *)mode {
  return [self valueForBinding:@"mode"];
}

- (void)setFetchSpecification:(EOFetchSpecification *)_fspec {
  [self setValue:_fspec forBinding:@"fetchSpecification"];
}

// additional accessors

- (NSString *)extendButtonLabel {
  NSString *label, *key;
  key = (self->isExtended)
    ? @"unextend"
    : @"extend";
  return ((label = [[self labels] valueForKey:key]))
    ? label
    : key;
}

- (NSString *)extendButtonImg {
  return (self->isExtended)
    ? @"up_icon.gif"
    : @"down_icon.gif";
}

- (NSString *)monthLabel {
  static NSArray *months = nil;
  int month;
  if (months == nil) {
    months =
    [[NSArray alloc] initWithObjects:
		       @"January", @"February", @"March", @"April",
		       @"May", @"June", @"July", @"August", @"September", 
		       @"October", @"November", @"December", nil];
  }
  month = [[self item] intValue];
  // TODO: range check!
  return [[self labels] valueForKey:[months objectAtIndex:(month - 1)]];
}

- (void)setSelMonth:(NSNumber *)_month {
  [self setMonth:[_month intValue]];
}
- (NSNumber *)selMonth {
  return [NSNumber numberWithInt:[self month]];
}

/* conditional */

- (BOOL)hasAccounts {
  return ((self->accounts == nil) || ([self->accounts count] == 0))
    ? NO : YES;
}
- (BOOL)hasPersons {
  return ((self->persons == nil) || ([self->persons count] == 0))
    ? NO : YES;
}
- (BOOL)hasTeams {
  return ((self->teams == nil) || ([self->teams count] == 0))
    ? NO : YES;
}
- (BOOL)hasResources {
  return ((self->resources == nil) || ([self->resources count] == 0))
    ? NO : YES;
}

/* direct action support */

- (int)nextDayNumber {
  return [[[self day] tomorrow] dayOfMonth];
}
- (int)nextDayMonth {
  return [[[self day] tomorrow] monthOfYear];
}
- (int)nextDayYear {
  return [[[self day] tomorrow] yearOfCommonEra];
}

- (int)lastDayNumber {
  return [[[self day] yesterday] dayOfMonth];
}
- (int)lastDayMonth {
  return [[[self day] yesterday] monthOfYear];
}
- (int)lastDayYear {
  return [[[self day] yesterday] yearOfCommonEra];
}

- (int)thisDayNumber {
  return [[NSCalendarDate date] dayOfMonth];
}
- (int)thisDayMonth {
  return [[NSCalendarDate date] monthOfYear];
}
- (int)thisDayYear {
  return [[NSCalendarDate date] yearOfCommonEra];
}

- (int)nextWeekNumber {
  NSCalendarDate *d;
  short woy, nowy;
  
  d = [[self weekStart] dateByAddingYears:0 months:0 days:7];
  
  woy  = [d weekOfYear];
  nowy = [d numberOfWeeksInYear];
  
  if (woy > nowy)
    woy = woy - nowy;
  
  return woy;
}
- (int)nextWeekYear {
  NSCalendarDate *d;
  short woy, nowy;
  
  d = [[self weekStart] dateByAddingYears:0 months:0 days:7];
  woy = [d weekOfYear];
  nowy = [d numberOfWeeksInYear];
  if (woy > nowy) 
    d = [d dateByAddingYears:0 months:0 days:6 hours:23 minutes:0 seconds:0];
  
  return [d yearOfCommonEra];
}

- (int)nextWeekMonth {
  int moy, nextYear, thisYear;
  NSCalendarDate *d;

  d = [self weekStart];
  thisYear = [d yearOfCommonEra];
  nextYear = [self nextWeekYear];
  if (nextYear != thisYear)
    moy = 1;
  else {
    d   = [[self weekStart] dateByAddingYears:0 months:0 days:7];
    moy = [d monthOfYear];
  }
  return moy;
}

- (int)lastWeekNumber {
  NSCalendarDate *d;
  short woy, nowy;
  
  d = [[self weekStart] dateByAddingYears:0 months:0 days:-7];

  woy = [d weekOfYear];
  nowy = [d numberOfWeeksInYear];
  if (woy > nowy)
    woy = woy - nowy;
  
  return woy;
}
- (int)lastWeekYear {
  NSCalendarDate *d;
  short woy, nowy;
  
  d = [[self weekStart] dateByAddingYears:0 months:0 days:-7];
  
  woy = [d weekOfYear];
  nowy = [d numberOfWeeksInYear];
  if (woy > nowy) 
    d = [d dateByAddingYears:0 months:0 days:6 hours:23 minutes:0 seconds:0];
  
  return [d yearOfCommonEra];
}

- (int)lastWeekMonth {
  NSCalendarDate *d;
  
  d = [[self weekStart] dateByAddingYears:0 months:0 days:-7];
  
  return [d monthOfYear];
}

- (int)thisWeekNumber {
  NSCalendarDate *d;
  d = [NSCalendarDate date];
  return [d weekOfYear];
}
- (int)thisWeekYear {
  NSCalendarDate *d;
  d = [NSCalendarDate date];
  return [d yearOfCommonEra];
}
- (int)thisWeekMonth {
  NSCalendarDate *d;
  d = [NSCalendarDate date];
  return [d monthOfYear];
}

- (int)serial {
  extern unsigned getpid(void);
  return getpid() + time(NULL);
}

- (int)nextMonthNumber {
  int m = [self month];
  return (m == 12) ? 1 : m + 1;
}

- (int)nextMonthYear {
  int m = [self month];
  int y = [self year];
  return (m == 12) ? y + 1 : y;
}

- (int)lastMonthNumber {
  int m = [self month];
  return (m == 1) ? 12 : m - 1;
}

- (int)lastMonthYear {
  int m = [self month];
  int y = [self year];
  return (m == 1) ? y - 1 : y;
}

- (int)thisMonthNumber {
  NSCalendarDate *d = [NSCalendarDate date];
  [d setTimeZone:[self timeZone]];
  return [d monthOfYear];
}

- (int)thisMonthYear {
  NSCalendarDate *d = [NSCalendarDate date];
  [d setTimeZone:[self timeZone]];
  return [d yearOfCommonEra];
}

- (int)nextYearNumber {
  return [self year] + 1;
}

- (int)lastYearNumber {
  return [self year] - 1;
}

- (int)thisYearNumber {
  return [[NSCalendarDate date] yearOfCommonEra];
}

- (NSDate *)timeZoneReferenceDate {
  NSString *m = [self mode];
  if ([m isEqualToString:@"dayoverview"])
    return [self day];
  else if ([m isEqualToString:@"weekoverview"]) 
    return [self weekStart];
  else if ([m isEqualToString:@"monthoverview"]) 
    return [NSCalendarDate dateWithYear:[self year] month:[self month] day:1
                           hour:0 minute:0 second:0 timeZone:[self timeZone]];
  
  return [NSCalendarDate dateWithYear:[self year] month:1 day:1
                         hour:0 minute:0 second:0 timeZone:[self timeZone]];
}

/* building fetchSpecification */

- (NSArray *)companiesToFetch {
  NSArray *comps;
  
  if (!self->isExtended) {
    if (![self->selectedCompany isKindOfClass:[NSString class]]) {
      comps = [NSArray arrayWithObject:
                       [self->selectedCompany valueForKey:@"globalID"]];
    }
    else
      comps = [NSArray array];
    if (self->fetchMeToo) {
      id me = [(id)[self session] activeAccount];
      comps = [comps arrayByAddingObject:[me valueForKey:@"globalID"]];
    }
  }
  else {
    comps = [NSArray array];
    if ((self->selectedAccounts != nil) &&
        ([self->selectedAccounts count] != 0)) {
      comps = [comps arrayByAddingObjectsFromArray:
                     [self->selectedAccounts valueForKey:@"globalID"]];
    }
    if ((self->selectedPersons != nil) &&
        ([self->selectedPersons count] != 0)) {
      comps = [comps arrayByAddingObjectsFromArray:
                     [self->selectedPersons valueForKey:@"globalID"]];
    }
    if ((self->selectedTeams != nil) &&
        ([self->selectedTeams count] != 0)) {
      comps = [comps arrayByAddingObjectsFromArray:
                     [self->selectedTeams valueForKey:@"globalID"]];
    }
  }
  return comps;
}

- (NSArray *)_fetchResourceGIDsForCategory:(NSString *)s {
  NSArray *tmp;

  tmp = [self runCommand:@"appointmentresource::extended-search",
	        @"fetchGlobalIDs", [NSNumber numberWithBool:YES],
                @"operator",       @"OR",
                @"category",       s,
                @"maxSearchCount", [NSNumber numberWithInt:1000], nil];
  return tmp;
}
- (NSArray *)_fetchResourceNamesForGIDs:(NSArray *)_gids {
  NSArray *tmp;
  
  tmp = [self runCommand:@"appointmentresource::get-by-globalid",
	        @"gids", _gids,
  	        @"attributes", [NSArray arrayWithObject:@"name"], nil];
  return tmp;
}

- (NSArray *)resourcesToFetch {
  // TODO: split up
  NSArray  *res;
  NSString *l;
  NSString *s;
  NSArray  *tmp;
  id  resSet;

  resSet = [NSMutableSet setWithCapacity:16];

  l = [[self labels] valueForKey:@"resCategory"];

  if (l == nil) l = @"resCategory";

  l = [NSString stringWithFormat:@"(%@)", l];

  [self setIsResCategorySelected:NO];
  
  if (!self->isExtended) {
    if ([self->selectedCompany isKindOfClass:[NSString class]]) {
      id s = self->selectedCompany;

      if ([s hasSuffix:l]) {
        s = [[s componentsSeparatedByString:@" ("] objectAtIndex:0];
	tmp = [self _fetchResourceGIDsForCategory:s];
        if ([tmp count] > 0)
	  tmp = [self _fetchResourceNamesForGIDs:tmp];
	
        if (tmp != nil && [tmp count] > 0) {
          [resSet addObjectsFromArray:[tmp valueForKey:@"name"]];
          [self setIsResCategorySelected:YES];
        }
        res = [resSet allObjects];
      }
      else
        res = [NSArray arrayWithObject:s];
    }
    else
      res = [NSArray array];
  }
  else {
    int i, cnt;

    for (i = 0, cnt = [self->selectedResources count]; i < cnt; i++) {
      id       r;

      r = [self->selectedResources objectAtIndex:i];
      s = [r valueForKey:@"name"];
      if ([s hasSuffix:l]) {
        s = [[s componentsSeparatedByString:@" ("] objectAtIndex:0];
	tmp = [self _fetchResourceGIDsForCategory:s];
        if ([tmp count] > 0)
	  tmp = [self _fetchResourceNamesForGIDs:tmp];
	
        if (tmp != nil && [tmp count] > 0) {
          [self setIsResCategorySelected:YES];
          [resSet addObjectsFromArray:[tmp valueForKey:@"name"]];
        }
      }
      else
        [resSet addObject:[r valueForKey:@"name"]];
    }
    res = [resSet allObjects];
  }
  return res;
}
  

- (EOFetchSpecification *)buildFetchSpecification {
  SkyAppointmentQualifier *q   = nil;
  EOFetchSpecification    *s   = nil;
  NSDictionary            *h   = nil;
  NSCalendarDate          *sD  = nil;
  NSCalendarDate          *eD  = nil;
  NSString                *m   = nil;
  static NSArray          *attrs = nil;
  NSArray                 *sO  = nil;
    
  if (attrs == nil) {
    attrs =
      [[NSArray alloc] initWithObjects:
                       @"title", @"location", @"aptType", 
		       @"startDate", @"endDate",
                       @"globalID", @"ownerId", @"accessTeamId",
                       @"permissions", @"resourceNames",
                       @"participants.companyId", @"participants.globalID",
                       @"participants.login",     @"participants.firstname",
                       @"participants.name",      @"participants.description",
                       @"participants.isTeam",    @"participants.isAccount",
                       nil];
  }
  
  q = [[[SkyAppointmentQualifier alloc] init] autorelease];
  m = [self mode];
  if ([m isEqualToString:@"dayoverview"]) {
    NSCalendarDate *d = [self day];
    sD = [d beginOfDay];
    eD = [d endOfDay];
  }
  else if ([m isEqualToString:@"weekoverview"]) {
    NSCalendarDate *ws = [self weekStart];
    sD = [ws beginOfDay];
    eD = [[ws dateByAddingYears:0 months:0 days:6] endOfDay];
  }
  else if ([m isEqualToString:@"monthoverview"]) {
    int dif;
    sD = [NSCalendarDate dateWithYear:[self year] month:[self month] day:1
                         hour:0 minute:0 second:0 timeZone:[self timeZone]];
    eD = [[sD lastDayOfMonth] endOfDay];
    
    //    dif = self->firstDayOfWeek - [sD dayOfWeek];
    dif = 1 - [sD dayOfWeek];
    dif = (dif > 0) ? dif - 7 : dif;
    sD  = [sD dateByAddingYears:0 months:0 days:dif];
    
    //    dif = self->firstDayOfWeek - [eD dayOfWeek] -1;
    dif = 0 - [eD dayOfWeek];
    dif = (dif < 0) ? dif + 7 : dif;
    eD  = [eD dateByAddingYears:0 months:0 days:dif];
  }
  else {
    sD = [NSCalendarDate dateWithYear:[self year] month:1 day:1
                         hour:0 minute:0 second:0 timeZone:[self timeZone]];
    eD = [[sD dateByAddingYears:1 months:0 days:-1] endOfDay];
  }

  [q setStartDate:sD];
  [q setEndDate:eD];
  [q setTimeZone:[self timeZone]];
  //  NSLog(@"Qualifier gets timeZone:%@",[self timeZone]);
  [q setCompanies:[self companiesToFetch]];
  [q setResources:[self resourcesToFetch]];
  [q setAptTypes:self->selectedAptTypes];
  
  h = [NSDictionary dictionaryWithObjectsAndKeys:
		      attrs, @"attributeKeys",
                    nil];
  
  sO = [NSArray arrayWithObject:
                [EOSortOrdering sortOrderingWithKey:@"startDate"
                                selector:EOCompareAscending]];
  
  s = [EOFetchSpecification fetchSpecificationWithEntityName:@"date"
                            qualifier:q
                            sortOrderings:sO];
  [s setHints:h];
  
  return s;
}

- (BOOL)needToReconfigure {
  if (self->reconfigure)
    return YES;

  if ((self->selAcCache == nil) && ([[self selectedAccounts] count] > 0))
    return YES;
  if ((self->selAcCache != nil) &&
      (![self->selAcCache isEqualToArray:[self selectedAccounts]])) 
    return YES;
  
  if ((self->selPerCache == nil) && ([[self selectedPersons] count] > 0))
    return YES;
  if ((self->selPerCache != nil) &&
      (![self->selPerCache isEqualToArray:[self selectedPersons]])) 
    return YES;

  if ((self->selTmCache == nil) && ([[self selectedTeams] count] > 0)) 
    return YES;
  if ((self->selTmCache != nil) &&
      (![self->selTmCache isEqualToArray:[self selectedTeams]])) 
    return YES;

  if ((self->selResCache == nil) && ([[self selectedResources] count] > 0))
    return YES;
  if ((self->selResCache != nil) &&
      (![self->selResCache isEqualToArray:[self selectedResources]])) 
    return YES;

  return NO;
}

- (void)writePanelItemsToDefaults {
  id uid;

  uid = [self->activeAccount valueForKey:@"companyId"];
  {
    NSEnumerator   *enumerator = nil;
    NSMutableArray *r          = nil;
    NSString       *n          = nil;
    NSString       *s          = nil;
    NSArray        *resNames   = nil;

    resNames = [self->resources valueForKey:@"name"];
    s = [[self labels] valueForKey:@"resCategory"];
    if (s == nil) s = @"resCategory";
    
    s = [NSString stringWithFormat:@"(%@)", s];
    r = [NSMutableArray arrayWithCapacity:8];
    
    enumerator = [resNames objectEnumerator];

    while ((n = [enumerator nextObject])) {
      if ([n hasSuffix:s]) {
        n = [[n  componentsSeparatedByString:@" ("] objectAtIndex:0];
        [r addObject:[NSString stringWithFormat:@"%@ (resCategory)", n]];
      }
      else 
        [r addObject:n];
    }

    [self runCommand:@"userdefaults::write",
          @"key",          @"scheduler_panel_resourceNames",
          @"value",        r,
          @"userdefaults", self->defaults,
          @"userId",       uid, nil];
  }
  {
    NSMutableArray *pIds       = nil;
    NSMutableArray *tIds       = nil;
    NSMutableArray *aIds       = nil;
    NSEnumerator   *enumerator = nil;
    id             obj         = nil;

    enumerator = [self->persons objectEnumerator];
    pIds = [[NSMutableArray alloc] init];
    tIds = [[NSMutableArray alloc] init];
    aIds = [[NSMutableArray alloc] init];

    while ((obj = [enumerator nextObject])) {
      [pIds addObject:[obj valueForKey:@"companyId"]];
    }
    enumerator = [self->accounts objectEnumerator];
    while ((obj = [enumerator nextObject])) {
      [aIds addObject:[obj valueForKey:@"companyId"]];
    }
    enumerator = [self->teams objectEnumerator];
    while ((obj = [enumerator nextObject])) {
      [tIds addObject:[obj valueForKey:@"companyId"]];
    }
    [self runCommand:@"userdefaults::write",
          @"key",          @"scheduler_panel_persons",
          @"value",        pIds,
          @"userdefaults", self->defaults,
          @"userId",       uid, nil];
    [self runCommand:@"userdefaults::write",
          @"key",          @"scheduler_panel_accounts",
          @"value",        aIds,
          @"userdefaults", self->defaults,
          @"userId",       uid, nil];
    [self runCommand:@"userdefaults::write",
          @"key",          @"scheduler_panel_teams",
          @"value",        tIds,
          @"userdefaults", self->defaults,
          @"userId",       uid, nil];
    [pIds release]; pIds = nil;
    [tIds release]; tIds = nil;
    [aIds release]; aIds = nil;
  }
}

- (void)reconfigure {
  //  NSLog(@"Reconfiguring dateSource!");
  [self setFetchSpecification:[self buildFetchSpecification]];
  self->reconfigure = NO;
  self->alreadyReconfigured = YES;

  if (self->isExtended) {
    [self writePanelItemsToDefaults];
  }
}

- (NSArray *)appendItems:(NSArray *)_items toArray:(NSArray *)_array {
  int i, cnt;
  id  obj;
  NSMutableArray *ma = [_array mutableCopy];
  if (ma == nil)
    ma = [[NSMutableArray alloc] init];

  i = 0; cnt = [_items count];
  
  while (i < cnt) {
    obj = [_items objectAtIndex:i++];
    if ((_array == nil) || ![_array containsObject:obj])
      [ma addObject:obj];
  }
  return AUTORELEASE(ma);
}

- (void)_initializePreSelectedItems {
  NSMutableArray *gids = nil;
  id             tmp   = nil;
  id             res   = nil;
  id             defs  = nil;
  int            i, cnt;

  defs = self->defaults;

  gids = [NSMutableArray arrayWithCapacity:16];
  {
    tmp = [defs arrayForKey:@"scheduler_panel_teams"];
    for (i = 0, cnt = [tmp count]; i < cnt; i++) {
      NSNumber      *pid;
      EOKeyGlobalID *gid;
        
      pid = [tmp objectAtIndex:i];
      gid = [EOKeyGlobalID globalIDWithEntityName:@"Team" 
                           keys:&pid keyCount:1 zone:NULL];        
      [gids addObject:gid];
    }
    if ([gids count] > 0) {
      res = [self runCommand:@"team::get-by-globalID",
                  @"gids",       gids,
                  @"attributes", [NSArray arrayWithObject:@"description"],
                  @"groupBy",    @"globalID",
                  nil];
      res = [res allValues];
      {
        NSArray *t;

        t = [res sortedArrayUsingKeyOrderArray:
                 [NSArray arrayWithObject:
                          [EOSortOrdering sortOrderingWithKey:@"description"
                                          selector:EOCompareAscending]]];
        RELEASE(self->selectedTeams);
        self->selectedTeams = [t mutableCopy];
        RELEASE(self->teams);
        self->teams = [self->selectedTeams copy];
      }
    }
  }
  [gids removeAllObjects];
  {
    tmp = [defs arrayForKey:@"scheduler_panel_accounts"];
    for (i = 0, cnt = [tmp count]; i < cnt; i++) {
      NSNumber      *pid;
      EOKeyGlobalID *gid;
        
      pid = [tmp objectAtIndex:i];
      gid = [EOKeyGlobalID globalIDWithEntityName:@"Person" 
                           keys:&pid keyCount:1 zone:NULL];        
      [gids addObject:gid];
    }
    if ([gids count] > 0) {
      res = [self runCommand:@"person::get-by-globalID",
                  @"gids",       gids,
                  @"attributes",
                  [NSArray arrayWithObjects:@"name", @"firstname",
                           @"isAccount", @"login", nil],
                  @"groupBy",    @"globalID",
                  nil];
      res = [res allValues];
      {
        NSArray *ac;

        ac = [res sortedArrayUsingKeyOrderArray:
               [NSArray arrayWithObjects:
                        [EOSortOrdering sortOrderingWithKey:@"name"
                                        selector:EOCompareAscending],
                        [EOSortOrdering sortOrderingWithKey:@"firstname"
                                        selector:EOCompareAscending],
                        nil]];
        RELEASE(self->selectedAccounts);
        self->selectedAccounts = [ac mutableCopy];
        RELEASE(self->accounts);
        self->accounts = [self->selectedAccounts copy];
      }
    }
  }
  [gids removeAllObjects];
  {
    tmp = [defs arrayForKey:@"scheduler_panel_persons"];
    for (i = 0, cnt = [tmp count]; i < cnt; i++) {
      NSNumber      *pid;
      EOKeyGlobalID *gid;
        
      pid = [tmp objectAtIndex:i];
      gid = [EOKeyGlobalID globalIDWithEntityName:@"Person" 
                           keys:&pid keyCount:1 zone:NULL];        
      [gids addObject:gid];
    }
    if ([gids count] > 0) {
      res = [self runCommand:@"person::get-by-globalID",
                  @"gids",       gids,
                  @"attributes",
                  [NSArray arrayWithObjects:@"name", @"firstname",
                           @"isAccount", @"login", nil],
                  @"groupBy",    @"globalID",
                  nil];
      res = [res allValues];
      {
        NSArray *ps;

        ps = [res sortedArrayUsingKeyOrderArray:
               [NSArray arrayWithObjects:
                        [EOSortOrdering sortOrderingWithKey:@"name"
                                        selector:EOCompareAscending],
                        [EOSortOrdering sortOrderingWithKey:@"firstname"
                                        selector:EOCompareAscending],
                        nil]];
        RELEASE(self->selectedPersons);
        self->selectedPersons = [ps mutableCopy];
        RELEASE(self->persons);
        self->persons = [self->selectedPersons copy];
      }
    }
  }
  tmp = [defs arrayForKey:@"scheduler_panel_resourceNames"];
    {
    NSEnumerator   *enumerator = nil;
    NSMutableArray *r          = nil;
    NSString       *n          = nil;
    NSString       *s          = nil;

    s = [[self labels] valueForKey:@"resCategory"];
    if (s == nil) s = @"resCategory";
    
    r = [NSMutableArray arrayWithCapacity:8];
    
    enumerator = [tmp objectEnumerator];

    while ((n = [enumerator nextObject])) {
      if ([n hasSuffix:@"(resCategory)"]) {
        n = [[n  componentsSeparatedByString:@" ("] objectAtIndex:0];
        [r addObject:[NSString stringWithFormat:@"%@ (%@)", n, s]];
      }
      else 
        [r addObject:n];
    }
    ASSIGN(self->resources, r);
  }

  {
    NSEnumerator   *enumerator = nil;
    NSMutableArray *r          = nil;
    NSString       *n          = nil;
    NSString       *s          = nil;

    s = [[self labels] valueForKey:@"resCategory"];
    if (s == nil) s = @"resCategory";
    
    r = [NSMutableArray arrayWithCapacity:8];
    
    enumerator = [tmp objectEnumerator];

    while ((n = [enumerator nextObject])) {
      NSMutableDictionary *rD;

      rD = [NSMutableDictionary dictionaryWithCapacity:1];

      if ([n hasSuffix:@"(resCategory)"]) {
        n = [[n  componentsSeparatedByString:@" ("] objectAtIndex:0];
        [rD setObject:
            [NSString stringWithFormat:@"%@ (%@)", n, s] forKey:@"name"];
      }
      else {
        [rD setObject:n forKey:@"name"];
      }
      [r addObject:rD];
    }    
    [self setSelectedResources:r];
    RELEASE(self->resources);
    self->resources = [self->selectedResources copy];
  }
}

- (NSArray *)distinctCategories:(NSArray *)_items {
  int i, cnt;
  id  obj;
  NSMutableArray *ma;
  NSString *l;

  ma = [[NSMutableArray alloc] init];
  l = [[self labels] valueForKey:@"resCategory"];

  if (l == nil) l = @"resCategory";
  
  i = 0; cnt = [_items count];
  
  while (i < cnt) {
    NSMutableDictionary *rD;
    NSString            *s;

    obj = [_items objectAtIndex:i++];

    s = [NSString stringWithFormat:@"%@ (%@)",
                  [obj valueForKey:@"category"], l];
    rD = [NSMutableDictionary dictionaryWithCapacity:1];
    [rD setObject:s forKey:@"name"];

    if (![ma containsObject:rD])
      [ma addObject:rD];
  }
  return AUTORELEASE(ma);
}

- (void)computeSearchString {
  if (!self->isExtended) {
    return;
  }
  if ((self->searchString == nil) ||
      ([self->searchString isEqualToString:@""])) {
    [self setTeams:self->selectedTeams];
    [self setResources:self->selectedResources];
    [self setAccounts:self->selectedAccounts];
    [self setPersons:self->selectedPersons];
  }
  else {
    id  res;
    int cnt;
    int max = self->maxSearchCount;

    // teams
    res = self->selectedTeams;
    cnt = (res) ? [res count] : 0;
    res = [self runCommand:@"team::extended-search",
                @"fetchGlobalIDs", [NSNumber numberWithBool:YES],
                @"operator",       @"OR",
                @"description",    self->searchString,
                @"maxSearchCount", [NSNumber numberWithInt:(max - cnt)],
                nil];
    if (res != nil) {
      res = [self runCommand:@"team::get-by-globalID",
                  @"gids",       res,
                  @"attributes", [NSArray arrayWithObject:@"description"],
                  @"groupBy",    @"globalID",
                  nil];
      res = [res allValues];
      {
        NSArray *t;

        t = [self appendItems:res toArray:self->selectedTeams];
        t = [t sortedArrayUsingKeyOrderArray:
               [NSArray arrayWithObjects:
                        [EOSortOrdering sortOrderingWithKey:@"description"
                                        selector:EOCompareAscending],
                        nil]];
        [self setTeams:t];
      }
    }
    // accounts
    res = self->selectedAccounts;
    cnt = (res) ? [res count] : 0;
    res = [self runCommand:@"account::extended-search",
                @"fetchGlobalIDs", [NSNumber numberWithBool:YES],
                @"operator",       @"OR",
                @"name",           self->searchString,
                @"firstname",      self->searchString,
                @"description",    self->searchString,
                @"login",          self->searchString,
                @"maxSearchCount", [NSNumber numberWithInt:(max - cnt)],
                nil];
    if (res != nil) {
      res = [self runCommand:@"person::get-by-globalID",
                  @"gids",       res,
                  @"attributes",
                  [NSArray arrayWithObjects:@"name", @"firstname",
                           @"isAccount", @"login", nil],
                  @"groupBy",    @"globalID",
                  nil];
      res = [res allValues];
      {
        NSArray *ac;

        ac = [self appendItems:res toArray:self->selectedAccounts];
        ac = [ac sortedArrayUsingKeyOrderArray:
               [NSArray arrayWithObjects:
                        [EOSortOrdering sortOrderingWithKey:@"name"
                                        selector:EOCompareAscending],
                        [EOSortOrdering sortOrderingWithKey:@"firstname"
                                        selector:EOCompareAscending],
                        nil]];
        [self setAccounts:ac];
      }
    }
    // persons
    res = self->selectedPersons;
    cnt = (res) ? [res count] : 0;
    res = [self runCommand:@"person::extended-search",
                @"fetchGlobalIDs",  [NSNumber numberWithBool:YES],
                @"operator",        @"OR",
                @"name",            self->searchString,
                @"firstname",       self->searchString,
                @"description",     self->searchString,
                @"login",           self->searchString,
                @"withoutAccounts", [NSNumber numberWithBool:YES],
                @"maxSearchCount",  [NSNumber numberWithInt:(max - cnt)],
                nil];
    if (res != nil) {
      res = [self runCommand:@"person::get-by-globalID",
                  @"gids",       res,
                  @"attributes",
                  [NSArray arrayWithObjects:@"name", @"firstname",
                           @"isAccount", @"login", nil],
                  @"groupBy",    @"globalID",
                  nil];
      res = [res allValues];
      {
        NSArray *p;

        p = [self appendItems:res toArray:self->selectedPersons];
        p = [p sortedArrayUsingKeyOrderArray:
               [NSArray arrayWithObjects:
                        [EOSortOrdering sortOrderingWithKey:@"name"
                                        selector:EOCompareAscending],
                        [EOSortOrdering sortOrderingWithKey:@"firstname"
                                        selector:EOCompareAscending],
                        nil]];
        [self setPersons:p];
      }
    }
    // resources
    {
      NSArray *r = nil;

      res = self->selectedResources;
      cnt = (res) ? [res count] : 0;
      res = [self runCommand:@"appointmentresource::extended-search",
                  @"fetchGlobalIDs",  [NSNumber numberWithBool:YES],
                  @"operator",        @"OR",
                  @"category",        self->searchString,
                  @"maxSearchCount",  [NSNumber numberWithInt:(max - cnt)],
                  nil];
      if (res != nil) {
        res = [self runCommand:@"appointmentresource::get-by-globalID",
                    @"gids",         res,
                    @"attributes",   [NSArray arrayWithObject:@"category"],
                    nil];
        r = [self distinctCategories:res];

        r = [self appendItems:r toArray:self->resources];

        if (r != nil) {
          r = [r sortedArrayUsingKeyOrderArray:
                 [NSArray arrayWithObjects:
                          [EOSortOrdering sortOrderingWithKey:@"name"
                                          selector:EOCompareAscending],
                          nil]];

          [self setResources:r];
        }
      }

      res = [self runCommand:@"appointmentresource::extended-search",
                  @"fetchGlobalIDs",  [NSNumber numberWithBool:YES],
                  @"operator",        @"OR",
                  @"name",            self->searchString,
                  @"maxSearchCount",  [NSNumber numberWithInt:(max - cnt)],
                  nil];
      if (res != nil) {
        res = [self runCommand:@"appointmentresource::get-by-globalID",
                    @"gids",         res,
                    @"attributes",   [NSArray arrayWithObject:@"name"],
                    nil];

        if (res != nil) {
          r = [res sortedArrayUsingKeyOrderArray:
                   [NSArray arrayWithObjects:
                            [EOSortOrdering sortOrderingWithKey:@"name"
                                            selector:EOCompareAscending],
                            nil]];
        }
        r = [self appendItems:res toArray:self->resources];
      }
      if (r != nil) {
        [self setResources:r];
      }
    }
  }
  [self setSearchString:@""];
}

- (BOOL)isTimeZoneLicensed {
  return YES;
}

- (id)reallyShow {
  self->reconfigure = YES;
  return [self show];
}

- (id)show {
  [self computeSearchString];

  if ([self needToReconfigure])
    [self reconfigure];
  // internal action --> no additional configuration
  self->alreadyReconfigured = YES;
  return nil;
}

- (id)extend {
  EOKeyGlobalID *gid;
  id me;

  me = [(id)[self session] activeAccount];
  gid = [me valueForKey:@"globalID"];

  me = [[self runCommand:@"person::get-by-globalID",
              @"gids",       [NSArray arrayWithObject:gid],
              @"attributes",
              [NSArray arrayWithObjects:@"companyId", @"name", @"firstname",
                       @"isAccount", @"login", @"globalID", nil],
              nil] lastObject];

  self->isExtended = (self->isExtended) ? NO : YES;

  if (self->isExtended) {
    [self _initializePreSelectedItems];
    [self reconfigure];
    return nil;
  }
  else {
    [self setSearchString:@""];
    [self setAccounts:nil];
    [self setSelectedAccounts:nil];
    [self setPersons:nil];
    [self setSelectedPersons:nil];
    [self setTeams:nil];
    [self setSelectedTeams:nil];
    [self setResources:nil];
    [self setSelectedResources:nil];
    [self setSearchString:@""];
  }
  return [self show];
}

- (void)appendToResponse:(WOResponse *)_response inContext:(WOContext *)_ctx {
#if 0  
  id tobj = nil;
  if ((tobj = [[self session] removeTransferObject]) != nil) {
    NSArray *ma;

    if ([tobj isKindOfClass:[NSString class]]) {
      // resource
      if (self->isExtended) {
        ma = [NSArray arrayWithObject:tobj];
        [self setSelectedResources:ma];
        [self setResources:ma];
        [self setAccounts:nil];
        [self setSelectedAccounts:nil];
        [self setPersons:nil];
        [self setSelectedPersons:nil];
        [self setTeams:nil];
        [self setSelectedTeams:nil];
        [self setSearchString:@""];
      }
      else {
        [self setSelectedCompany:tobj];
      }
    }
    else if ([tobj isKindOfClass:[NSNumber class]]) {
      // company
      EOKeyGlobalID *gid;
      id            result;
      gid = [EOKeyGlobalID globalIDWithEntityName:@"Person" 
                           keys:&tobj keyCount:1 zone:NULL];        
      result = [self runCommand:@"person::get-by-globalID",
                     @"gids",       [NSArray arrayWithObject:gid],
                     @"attributes",
                     [NSArray arrayWithObjects:@"name", @"firstname",
                              @"isAccount", @"login", nil],
                     @"groupBy",    @"globalID",
                     nil];
      result = [[result allValues] lastObject];

      if (result && [result count] > 0) {
        if (self->isExtended) {
          ma = [NSArray arrayWithObject:result];
          [self setSelectedResources:nil];
          [self setResources:nil];
          [self setTeams:nil];
          [self setSelectedTeams:nil];
          [self setSearchString:@""];
          if ([[result valueForKey:@"isAccount"] boolValue]) {
            [self setSelectedAccounts:ma];
            [self setAccounts:ma];
            [self setPersons:nil];
            [self setSelectedPersons:nil];
          } else {
            [self setAccounts:nil];
            [self setSelectedAccounts:nil];
            [self setSelectedPersons:ma];
            [self setPersons:ma];
          }
        }
        else {
          [self setSelectedCompany:result];
        }
      }
    }
    self->alreadyReconfigured = NO;
  }
#endif

  RELEASE(self->selAcCache);
  self->selAcCache  = [self->selectedAccounts copy];
  RELEASE(self->selPerCache);
  self->selPerCache = [self->selectedPersons copy];
  RELEASE(self->selTmCache);
  self->selTmCache  = [self->selectedTeams copy];
  RELEASE(self->selResCache);
  self->selResCache = [self->selectedResources copy];

  if (!self->alreadyReconfigured) 
    [self reconfigure];
  self->alreadyReconfigured = NO;

  [super appendToResponse:_response inContext:_ctx];
}

@end

