/*
  Copyright (C) 2000-2003 SKYRIX Software AG

  This file is part of OGo

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/
// $Id: SkyInlineDayOverview.m,v 1.3 2003/09/03 14:13:57 martin Exp $

#include "SkyInlineAptDataSourceView.h"
#include <OGoScheduler/SkyAptDataSource.h>
#include "SkyAppointmentFormatter.h"
#import <Foundation/NSString.h>

@class NSCalendarDate;

@interface SkyInlineDayOverview : SkyInlineAptDataSourceView
{
@protected
  NSCalendarDate *day;
  id             intervalDataSource;
  
  // config
  NSTimeInterval interval;
  NSTimeInterval dayStart;
  NSTimeInterval dayEnd;

  NSArray        *currentApts;
  NSArray        *moreDayApts;
  NSArray        *allDayDates;
}

@end

#include <OGoFoundation/WOComponent+config.h>
#include <OGoFoundation/LSWSession.h>
#include <OGoFoundation/LSWNavigation.h>
#include <OGoFoundation/WOComponent+Commands.h>
#include <LSFoundation/LSCommandContext.h>
#include <NGExtensions/NGExtensions.h>
#include <NGExtensions/EOFilterDataSource.h>
#import <NGMime/NGMimeType.h>
#import <EOControl/EOControl.h>
#import <EOAccess/EOAccess.h>
#import <NGObjWeb/NGObjWeb.h>

@implementation SkyInlineDayOverview

- (id)init {
  if ((self = [super init])) {
    NSNotificationCenter *nc = nil;

    nc = [NSNotificationCenter defaultCenter];
    self->moreDayApts = nil;
    self->allDayDates = nil;
    self->currentApts = nil;
    self->printMode   = NO;
    [nc addObserver:self selector:@selector(noteChange:)
        name:@"SkyDataSourceWillClear" object:nil];
  }
  return self;
}

#if !LIB_FOUNDATION_BOEHM_GC
- (void)dealloc {
  [[NSNotificationCenter defaultCenter] removeObserver:self];
  RELEASE(self->intervalDataSource);
  RELEASE(self->day);
  RELEASE(self->moreDayApts);
  RELEASE(self->currentApts);
  RELEASE(self->allDayDates);
  [super dealloc];
}
#endif

- (void)syncAwake {
  [super syncAwake];
  self->interval =
    [[(id)[(id)[self session] userDefaults]
          objectForKey:@"scheduler_dayoverview_timeinterval"]
                     doubleValue];
  self->dayStart =
    [[(id)[(id)[self session] userDefaults]
          objectForKey:@"scheduler_dayoverview_daystart"]
                     doubleValue];
  self->dayEnd =
    [[(id)[(id)[self session] userDefaults]
          objectForKey:@"scheduler_dayoverview_dayend"]
                     doubleValue];
  RELEASE(self->moreDayApts); self->moreDayApts = nil;
  RELEASE(self->allDayDates); self->allDayDates = nil;
  RELEASE(self->currentApts); self->currentApts = nil;
  [self setCurrentDate:nil];
}

- (void)noteChange:(id)_note {
  RELEASE(self->moreDayApts); self->moreDayApts = nil;
  RELEASE(self->currentApts); self->currentApts = nil;
  RELEASE(self->allDayDates); self->allDayDates = nil;
}

// accessors

/* filter datasource support */

- (EOQualifier *)intervalQualifier {
  EOQualifier *q;
  NSCalendarDate *s, *e;
  NSCalendarDate *begin, *end1, *end2;

  s = [self currentDate];
  e = [s dateByAddingYears:0 months:0 days:0 hours:0 minutes:0
         seconds:(int)self->interval];
  
  begin = [s beginOfDay];
  end1  = [[s endOfDay] dateByAddingYears:0 months:0 days:0
                        hours:0 minutes:-1 seconds:0]; // 23:58:59
  end2  = [begin dateByAddingYears:0 months:0 days:1
                 hours:0 minutes:0 seconds:0]; // 24:00:00
  
  q = [EOQualifier qualifierWithQualifierFormat:
                   @"(startDate > %@ OR startDate = %@)"
                   @" AND startDate < %@ AND NOT "
                   @"(startDate = %@ AND endDate > %@ AND endDate < %@ )",
                   s, s, e, begin, end1, end2];
  return q;
}

- (EOQualifier *)allDayAptsQualifier {
  EOQualifier *q;
  NSCalendarDate *begin, *end1, *end2;

  begin = [self->day beginOfDay];
  end1  = [[begin endOfDay] dateByAddingYears:0 months:0 days:0
                            hours:0 minutes:-1 seconds:0]; // 23:58:59
  end2  = [begin dateByAddingYears:0 months:0 days:1
                 hours:0 minutes:0 seconds:0]; // 24:00:00
  q = [EOQualifier qualifierWithQualifierFormat:
                   @"startDate = %@ AND endDate > %@ AND endDate < %@",
                   begin, end1, end2];
  return q;
}

- (EOQualifier *)moreDayAptsQualifier {
  EOQualifier *q;
  NSCalendarDate *s;

  s = [self->day beginOfDay];
  q = [EOQualifier qualifierWithQualifierFormat:
                   @"startDate < %@ AND endDate > %@",
                   s, s];
  return q;
}

- (void)setDataSource:(id)_ds {
  if (self->dataSource != _ds) {
    [super setDataSource:_ds];
    RELEASE(self->intervalDataSource);
    //    self->intervalDataSource =
    //      [[EOFilterDataSource alloc] initWithDataSource:_ds];
    self->intervalDataSource =
      [[EOFilterDataSource alloc] initWithDataSource:[self cacheDataSource]];
    [self->intervalDataSource setSortOrderings:[self sortOrderings]];
  }
}

- (id)intervalDataSource {
  return self->intervalDataSource;
}

- (void)setDay:(NSCalendarDate *)_day {
  ASSIGN(self->day,_day);
}
- (NSCalendarDate *)day {
  return self->day;
}

- (void)setCurrentTime:(NSCalendarDate *)_time {
  if (![_time isEqual:self->currentDate]) {
    RELEASE(self->currentApts); self->currentApts = nil;
    [self setCurrentDate:_time];
  }
}
- (NSCalendarDate *)currentTime {
  return [self currentDate];
}

- (NSArray *)moreDayApts {
  if (self->moreDayApts == nil) {
    NSArray *ma;

    [self->intervalDataSource setAuxiliaryQualifier:[self moreDayAptsQualifier]];
    ma = [self->intervalDataSource fetchObjects];
    ASSIGN(self->moreDayApts, ma);
  } 
  return self->moreDayApts;
}

- (NSArray *)allDayApts {
  if (self->allDayDates == nil) {
    NSArray *aa;

    [self->intervalDataSource setAuxiliaryQualifier:
         [self allDayAptsQualifier]];
    aa = [self->intervalDataSource fetchObjects];
    ASSIGN(self->allDayDates, aa);
  }
  return self->allDayDates;
}

/* month browser support */

- (NSString *)browserFontColor {
  if ([self->browserDate isDateOnSameDay:self->day])
    return @"#FF0000";
  if (!self->browserDateInMonth)
    return @"#555555";
  return @"#000000";
}

// conditional

- (BOOL)hasMoreDayApts {
  return ([[self moreDayApts] count] > 0) ? YES : NO;
}
- (BOOL)hasAllDayApts {
  return ([[self allDayApts] count] > 0) ? YES : NO;
}

// additional accessors

- (int)secondsFromTimeInterval:(NSTimeInterval)_interval {
  return [[NSNumber numberWithDouble:_interval] intValue];
}

- (NSArray *)listOfTimes {
  NSArray        *apts;
  NSCalendarDate *firstDate;
  NSCalendarDate *lastDate;
  NSCalendarDate *date;
  double         helper;
  NSMutableArray *times;

  apts = [[self cacheDataSource] fetchObjects];
  //  apts = [self->dataSource fetchObjects];

  // hope that array is time sorted

  firstDate = nil;
  
  if ([apts count] > 0) {
    int            cnt = 0;
    id             apt;
    NSCalendarDate *d;
    BOOL           hasOneDayApts = NO;
    NSArray        *ma;

    firstDate = [self->day beginOfDay];
    ma        = [self moreDayApts];

    while (cnt < [apts count]) {
      apt = [apts objectAtIndex:cnt++];
      d   = [apt valueForKey:@"startDate"];

      if (!d)
        continue;

      // ignore all day apts
      if ([self isThisAllDayApt:apt])
        continue;
      // to jump over more day apts
      if ([ma containsObject:apt])
        continue;

      firstDate     = d;
      hasOneDayApts = YES;
      break;
    }
    if (!hasOneDayApts) {
      // no one-day apts
      firstDate = [self->day endOfDay];
      lastDate  = [self->day beginOfDay];
    }
    else {
      lastDate = [[apts lastObject] valueForKey:@"startDate"];
    }
  }
  else {
    // to force the next condition to proceed
    firstDate = [self->day endOfDay];
    lastDate  = [self->day beginOfDay];
  }

  helper = self->dayStart;
  if (helper <= ([firstDate hourOfDay]*60 + [firstDate minuteOfHour])) {
    firstDate = [firstDate beginOfDay];
    firstDate =
      [NSCalendarDate dateWithTimeIntervalSince1970:
                      [firstDate timeIntervalSince1970] + helper*60];
  }
  else {
    date = [firstDate beginOfDay];
    helper = [firstDate hourOfDay]*3600 + [firstDate minuteOfHour]*60 +
             [firstDate secondOfMinute];
    
    while (([date hourOfDay]*3600 + [date minuteOfHour]*60 +
            [date secondOfMinute] + self->interval) <= helper)
      {
        unsigned seconds = (unsigned)self->interval;

        date = [date dateByAddingYears:0 months:0 days:0 hours:0 minutes:0
                     seconds:seconds];
      }
    firstDate = date;
  }

  helper = self->dayEnd;
  if (helper > ([lastDate hourOfDay]*60 + [lastDate minuteOfHour])) {
    lastDate = [lastDate beginOfDay];
    lastDate =
      [NSCalendarDate dateWithTimeIntervalSince1970:
                      [lastDate timeIntervalSince1970] + helper*60];
  }

  [firstDate setTimeZone:[self->day timeZone]];
  [lastDate setTimeZone:[self->day timeZone]];
  
  times = [NSMutableArray array];
    
  while ((firstDate != nil) &&
         (([firstDate earlierDate:lastDate] == firstDate) ||
          ([firstDate isEqual:lastDate])))
    {
      unsigned seconds = (unsigned)self->interval;

      [times addObject:firstDate];
      firstDate = [firstDate dateByAddingYears:0 months:0 days:0 hours:0
                             minutes:0 seconds:seconds];
    }

  return times;
}

- (NSArray *)currentAppointments {
  NSArray *apts;
  
  if (self->currentApts == nil) {
    if (self->currentDate == nil) {
      apts = [self moreDayApts];
    }
    else {
      [self->intervalDataSource setAuxiliaryQualifier:[self intervalQualifier]];
      apts = [self->intervalDataSource fetchObjects];
    }
    ASSIGN(self->currentApts, apts);
  }
  return self->currentApts;
}

- (BOOL)hasCurrentAppointments {
  NSArray *apts;
  
  apts = [self currentAppointments];
  return ((apts!= nil) && ([apts count] > 0)) ? YES : NO;
}

- (int)rowspanForCurrentTime {
  int rowspan;
  rowspan = [[self currentAppointments] count];
  return (rowspan > 0) ? rowspan : 1;
}

- (int)rowspanForMoreDayApts {
  int rowspan;
  rowspan = [[self moreDayApts] count];
  return (rowspan > 0) ? rowspan : 1;
}

- (int)rowspanForAllDayApts {
  int rowspan;
  rowspan = [[self allDayApts] count];
  return (rowspan > 0) ? rowspan : 1;
}

- (NSString *)labelOfCurrentTime {
  NSString *format;
  format = [self showAMPMDates] ? @"%I:%M %p" : @"%H:%M";
  return [self->currentDate descriptionWithCalendarFormat:format];
}

- (NSCalendarDate *)referenceDateForFormatter {
  return self->day;
}

- (NSFormatter *)aptFullInfoFormatter {
  id f;
  
  f = [super aptFullInfoFormatter];
  [f setRelationDate:self->day];

  return f;
}

- (NSFormatter *)aptContentFormatter {
  SkyAppointmentFormatter *f;
  NSMutableString         *format;
  id                      res, loc;

  format = [NSMutableString stringWithCapacity:128];

  res = [self->appointment valueForKey:@"resourceNames"];
  loc = [self->appointment valueForKey:@"location"];
  
  if (loc != nil && [loc length] > 0 && ![loc isEqualToString:@" "])
    [format appendString:[NSString stringWithFormat:@"; %%L"]];

  [format appendString:[NSString stringWithFormat:@"; %%P"]];

  if (res != nil && [res length] > 0 && ![res isEqualToString:@" "])
    [format appendString:[NSString stringWithFormat:@"; %%R"]];
            
  f = [SkyAppointmentFormatter formatterWithFormat:format];
  [f setShowFullNames:[self showFullNames]];
  return f;
}

- (NSFormatter *)aptTimeFormatter {
  SkyAppointmentFormatter *format;
  
  format = [SkyAppointmentFormatter formatterWithFormat:@"%S - %E; "];
  [format setRelationDate:self->day];
  if ([self showAMPMDates]) [format switchToAMPMTimes:YES];
  return format;
}

- (NSString *)currentDayInfo {
  NSString *info;

  [self setCurrentDate:self->day];
  info = [self holidayInfo];
  [self setCurrentDate:nil];
  
  return info;
}

- (BOOL)hasHolidays {
  return ([[self currentDayInfo] length]) ? YES : NO;
}


// key/value coding

- (void)takeValue:(id)_val forKey:(id)_key {
  if ([_key isEqualToString:@"day"])
    [self setDay:_val];
  else
    [super takeValue:_val forKey:_key];
}

- (id)valueForKey:(id)_key {
  if ([_key isEqualToString:@"day"])
    return [self day];
  return [super valueForKey:_key];
}

@end /* SkyInlineDayOverview */
