/*
  Copyright (C) 2000-2004 SKYRIX Software AG

  This file is part of OGo

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/
// $Id: SkyPrintMonthOverview.m,v 1.4 2004/01/20 01:36:18 helge Exp $

#include "SkyInlineAptDataSourceView.h"

@class NSTimeZone;

@interface SkyPrintMonthOverview : SkyInlineAptDataSourceView
{
  int dayOfWeek;
  int year;
  int month;
  int weekOfYear;

  NSTimeZone *tz;
}
@end

#include "SkyAppointmentFormatter.h"
#include "common.h"
#include <OGoScheduler/SkyAptDataSource.h>
#include <OGoFoundation/LSWSession.h>

@implementation SkyPrintMonthOverview

- (void)dealloc {
  [self->tz release];
  [super dealloc];
}

/* accessors */

- (BOOL)isInMonth {
  return [[self currentDate] monthOfYear] == self->month;
}

- (void)setMonth:(int)_m {
  self->month = _m;
}
- (int)month {
  return self->month;
}

- (void)setYear:(int)_y {
  self->year = _y;
}
- (int)year {
  return self->year;
}

- (void)setWeekOfYear:(int)_w {
  self->weekOfYear = _w;
}
- (int)weekOfYear {
  return self->weekOfYear;
}

- (void)setTimeZone:(NSTimeZone *)_tz {
  ASSIGN(self->tz,_tz);
}
- (NSTimeZone *)timeZone {
  return self->tz;
}

/* additional accessors */

- (NSString *)weekDayString {
  NSArray *days =
    [NSArray arrayWithObjects:@"Sunday", @"Monday", @"Tuesday",
             @"Wednesday", @"Thursday", @"Friday", @"Saturday", nil];
  return [days objectAtIndex:self->dayOfWeek];
}


- (NSString *)monthString {
  NSArray *months =
    [NSArray arrayWithObjects:@"January", @"February", @"March", @"April",
             @"May", @"June", @"July", @"August", @"September", @"October",
             @"November", @"December", nil];

  return [months objectAtIndex:(self->month - 1)];
}

- (BOOL)appointmentViewAccessAllowed {
  NSString *perms;
  
  if ((perms = [self->appointment valueForKey:@"permissions"]))
    return [perms rangeOfString:@"v"].length > 0 ? YES : NO;
  
  return [[self->appointment valueForKey:@"isViewAllowed"] boolValue];
}

- (NSFormatter *)aptFormatter {
  SkyAppointmentFormatter *f;
  NSMutableString         *format;
  id                      res, loc;
  int                     len;

  format = [NSMutableString stringWithCapacity:128];

  len = [self appointmentViewAccessAllowed]
    ? [[self->appointment valueForKey:@"title"] length]
    : 0;

  res = [self->appointment valueForKey:@"resourceNames"];
  loc = [self->appointment valueForKey:@"location"];

#if DEBUG && 0
  NSAssert([loc isNotNull],
           @"'location' key of %@<%@> is NSNull",
           self->appointment, NSStringFromClass([self->appointment class]));
  NSAssert([res isNotNull],
           @"'resourceNames' key of %@<%@> is NSNull",
           self->appointment, NSStringFromClass([self->appointment class]));
#endif
  
  [format appendString:
          [NSString stringWithFormat:@"<B><I>%%S - %%E</I></B>, %%%dT", len]];

  if (loc != nil && [loc length] > 0 && ![loc isEqualToString:@" "])
    [format appendString:[NSString stringWithFormat:@"\n%%L"]];

  [format appendString:[NSString stringWithFormat:@"\n%%P"]];
  
  if (res != nil && [res length] > 0 && ![res isEqualToString:@" "])
    [format appendString:[NSString stringWithFormat:@"\n%%R"]];
            
  f = [SkyAppointmentFormatter formatterWithFormat:format];

  [f setRelationDate:self->currentDate];
  [f setShowFullNames:[self showFullNames]];

  return f;
}

- (void)setDayOfWeek:(int)_day {
  self->dayOfWeek = _day;
}
- (int)dayOfWeek {
  return self->dayOfWeek;
}

/* kvs */
- (void)takeValue:(id)_value forKey:(id)_key {
  if ([_key isEqualToString:@"month"])
    [self setMonth:[_value intValue]];
  else if ([_key isEqualToString:@"year"])
    [self setYear:[_value intValue]];
  else if ([_key isEqualToString:@"timeZone"])
    [self setTimeZone:_value];
  else
    [super takeValue:_value forKey:_key];
}

@end /* SkyPrintMonthOverview */
