/*
  Copyright (C) 2000-2003 SKYRIX Software AG

  This file is part of OGo

  OGo is free software; you can redistribute it and/or modify it under
  the terms of the GNU Lesser General Public License as published by the
  Free Software Foundation; either version 2, or (at your option) any
  later version.

  OGo is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
  License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with OGo; see the file COPYING.  If not, write to the
  Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
  02111-1307, USA.
*/
// $Id: DirectAction+Generic.m,v 1.1 2003/10/29 21:51:06 jan Exp $

#include "DirectAction.h"
#include "common.h"
#include "EOControl+XmlRpcDirectAction.h"

@implementation DirectAction(Generic)

/* access */


/*
  Returns true or false if the object with url _access has access (_op (rw..))
  to the objects referred as urls in _objs
*/
  

- (id)access_operationAllowedOnObjectsForAccessAction:(id)_op
  :(id)_objs:(id)_access
{
  id      docManager, accessId;
  NSArray *objIds;

  if (!_objs || !_op)
    return nil;
  
  docManager = [[self commandContext] documentManager];
  objIds     = [docManager globalIDsForURLs:_objs];
  if (_access == nil)
    accessId = [[[self commandContext] valueForKey:LSAccountKey] globalID];
  else
    accessId   = [[docManager globalIDsForURLs:[NSArray arrayWithObject:
                                                        _access]] lastObject];

  return [NSNumber numberWithBool:
                   [[[self commandContext] accessManager]
                           operation:_op
                           allowedOnObjectIDs:objIds
                           forAccessGlobalID:accessId]];
}

/*
  Set the access given access right (_op) to the object referred as url
  in _obj to the object _access (also referred as url. Returns true or false
  on success or failed.
*/

- (id)access_setOperationOnObjectForAccessAction:(id)_op:(id)_obj:(id)_access
{
  id docManager, objId, accessId;

  if (!_op || !_obj)
    return nil;
  
  docManager = [[self commandContext] documentManager];
  objId      = [[docManager globalIDsForURLs:[NSArray arrayWithObject:_obj]]
                            lastObject];
  if (_access == nil)
    accessId = [[[self commandContext] valueForKey:LSAccountKey] globalID];
  else
    accessId   = [[docManager globalIDsForURLs:
                              [NSArray arrayWithObject:_access]] lastObject];

  return [NSNumber numberWithBool:
                   [[[self commandContext] accessManager]
                           setOperation:_op
                           onObjectID:objId
                           forAccessGlobalID:accessId]];
}

@end /* DirectAction(Generic) */
