// $Id: SxProjectsFolder.m,v 1.8 2003/12/09 17:53:06 helge Exp $

#include "SxProjectsFolder.h"
#include "SxMapEnumerator.h"
#include "common.h"

// TODO: we should use commands only
#include <OGoProject/SkyProjectDataSource.h>

@implementation SxProjectsFolder

static BOOL debugOn = YES;

- (void)dealloc {
  [self->projectNames release];
  [super dealloc];
}

/* child names */

- (NSArray *)toManyRelationshipKeys {
  LSCommandContext *cmdctx;
  NSArray          *names, *projects;

  if (self->projectNames)
    return self->projectNames;
  
  if ((cmdctx = [self commandContextInContext:nil]) == nil) {
    [self logWithFormat:@"no command context!"];
    return nil;
  }
  
  projects = [cmdctx runCommand:@"person::get-projects",
                       @"object", [cmdctx valueForKey:LSAccountKey], 
                     nil];
  names = [projects valueForKey:@"number"];
  
  self->projectNames = 
    [[names sortedArrayUsingSelector:@selector(compare:)] copy];
  
  [self debugWithFormat:@"  fetched %i project names", [names count]];
  return self->projectNames;
}

- (NSEnumerator *)davChildKeysInContext:(id)_ctx {
  [self debugWithFormat:@"davChildKeysInContext:"];
  return [[self toManyRelationshipKeys] objectEnumerator];
}

- (EODataSource *)rawContentDataSourceInContext:(id)_ctx {
  SkyProjectDataSource *ds;
  id cmdctx;

  if ((cmdctx = [self commandContextInContext:_ctx]) == nil) {
    [self logWithFormat:@"missing command context."];
    return nil;
  }
  ds = [[[SkyProjectDataSource alloc] initWithContext:cmdctx] autorelease];
  if (ds == nil) {
    [self logWithFormat:@"could not create project datasource!"];
    return nil; 
  }
  // TODO: setup some useful spec?
  return ds;
}

/* name lookup */

- (BOOL)isProjectName:(NSString *)_name inContext:(id)_ctx {
  if ([_name length] == 0) return NO;
  return [[self toManyRelationshipKeys] containsObject:_name];
}

- (id)projectFolder:(NSString *)_name inContext:(id)_ctx {
  id folder;
  
  folder = [[NSClassFromString(@"SxProjectFolder") alloc] 
             initWithName:_name inContainer:self];
  return [folder autorelease];
}

- (id)lookupName:(NSString *)_key inContext:(id)_ctx acquire:(BOOL)_flag {
  id o;
  
  if ([_key length] == 0) return nil;
  
  if ([_key isEqualToString:@"getIDsAndVersions"])
    return [self getIDsAndVersionsAction:_ctx];

  /* this is placed above isProjectName to avoid fetching of the names */
  if ((o = [super lookupName:_key inContext:_ctx acquire:_flag]))
    return o;
  
  if ([self isProjectName:_key inContext:_ctx])
    return [self projectFolder:_key inContext:_ctx];
  
  return nil;
}

/* actions */

- (NSString *)defaultMethodNameInContext:(id)_ctx {
  return @"view";
}

/* debugging */

- (BOOL)isDebuggingEnabled {
  return debugOn;
}

- (NSString *)description {
  NSMutableString *ms = [NSMutableString stringWithCapacity:128];
  
  [ms appendFormat:@"<0x%08X[%@]:", self, NSStringFromClass([self class])];
  
  if (self->projectNames)
    [ms appendFormat:@" #projects=%i", [self->projectNames count]];
  
  [ms appendString:@">"];
  return ms;
}

@end /* SxProjectsFolder */
